/*
   -----------------------------------------------------------------------------
   GenerateQR.prg
   Posted by Maurizio S. in the dBASE Newsgroups
   Modified by: Ken Mayer
   Date.......: 10/28/2016
   Notes......: I did this because I wanted to look at something completely
                different from what I've been tinkering with. I found the
                idea fascinating (and I've always liked the idea of QR
                Codes -- when I taught I used them on my syllabus to
                provide contact info for my students).
                
                There are a TON of notes below.
   Usage......: set procedure to GenerateQR.prg
                cImg = "myfolder/Name_of_file.png" // generates a .png file
                cStr = "string to convert to QR Code" // see notes below
                nSize = 75 // create 75 x 75 pixel QR code ...
                ? GenerateQR( cImg, cStr, nSize )
   
                The image specified by cImg is created, you can then
                open it, use it in a report, whatever you need to do.
                
   OPTIONS:
   A bit of digging around on the web:
      http://www.cognation.net/QR/google_api.htm
      
   For a QR Code with plain text, just pass the text in the 
   cStr parameter.
   
   // on an iPhone this will bring up a contact card, and give the option
   // to create a new contact. Assumes a home address, but that's okay.
   For a "MECARD" -- similar to vcard, but less options, you can pass detailed
   information by using a string such as:
       cStr = "MECARD:N:<name>;ADR:<street address, city state zip>;"+;
              "TEL:<phone>;EMAIL:somewhere@somewhere.com;"+;
              "URL:http://www.somewhere.com;NOTE:<Some text>"
   Other details include:
       Multiple phone numbers by adding a comma and a second number:
           TEL:phone1,TEL:phone2
           
   For a VCARD (more traditional, also more options):
      http://fisherwebdev.com/wordpress/2010/05/21/vcards-as-qr-codes-with-google-image-chart-api/
   There's more information about VCard formats here -- I didn't test every
   possible permutation:
      https://en.wikipedia.org/wiki/VCard
   
       This mostly works pretty well. On the iPhone "CELL" shows up as "mobile" ...
       Read the options carefully ...:
       
       cStr = "BEGIN:VCARD"        // REQUIRED
       cStr += "%0A" // new line
       cStr += "VERSION:3.0"       // REQUIRED
       cStr += "%0A" // new line
       cStr += "N:Last;First"      // REQUIRED: lastname semicolon firstname
       cStr += "%0A" // new line
       cStr += "FN:First Last"     // REQUIRED: Formatted Name
       cStr += "%0A" // new line
       cStr += "ORG:My Business Name" // organization or business
       cStr += "%0A" // new line
       cStr += "TITLE:My Job Title"  // title in organization
       cStr += "%0A" // new line
       cStr += "TEL;TYPE=WORK,VOICE:(999) 555-1212"
       cStr += "%0A" // new line
       cStr += "TEL;TYPE=CELL:(999) 555-1515"
       cStr += "%0A" // new line
       cStr += "TEL;TYPE=HOME,VOICE:(999) 555-1213"
       cStr += "%0A" // new line
       cStr += "ADR;TYPE=WORK:;;Street Address;City;ST;ZIP;Country" // Country = "United States of America"
       cStr += "%0A" // new line
       cStr += "LABEL;TYPE=WORK:Street Address\nCity, ST ZIP\nCountry"
       cStr += "%0A" // new line
       cStr += "ADR;TYPE=HOME:;;Street;City;ST;ZIP;Country"
       cStr += "%0A" // new line
       cStr += "LABEL;TYPE=HOME:Street Address\nCity, ST ZIP\nCountry"
       cStr += "%0A" // new line
       cStr += "EMAIL;TYPE=PREF,INTERNET:someone@somewhere.com" // use a real address obviously
       cStr += "%0A" // new line
       cStr += "URL:http://mybusiness.somewhere.com" // use a real site obviously
       cStr += "%0A" // new line
       cStr += "NOTE:Supplemental information -- please visit our company website" // use a real site obviously
       cStr += "%0A" // new line
       cStr += "END:VCARD"            // REQUIRED
           
   // I tested my own number, it did bring up my contact information 
   // as a contact card, and you can click the little 'phone' 
   // icon there and it will dial the number:
   To have a phone dial a phone number:
      cStr = "tel:phone"
      
   // This brought up the browser (Safari) on my iPhone and loaded
   // a website (I used a valid site):
   A Web address:
      cStr = "http://www.somewhere.com"
   
   // Created a "note", but in the upper right corner the "send" icon
   // appears, if you tap that, you are given options for where to
   // send it to, and one of them is the obvious email button.
   // Oddly, it doesn't put the address in the "to" field, it places
   // it in the body of the message   
   Start an EMail message:
      cStr = "email@somewhere.com" // proper email address
      
   // The first didn't work for me, it crashed the app ("QR Reader")
   Send a text message via SMS:
      cStr = "sms:number"
   // This however brings up a screen with an icon at the top (on iPhone,
   // square with arrow up ... -- a "send" icon)
   // to send the message, if you tap it, you are asked 
   // if you want to send the message ("Send Mesage", "Share" and 
   // "Cancel" options), if you click "Send Message" it starts 
   // the iMessage app, with the text ... if you send that, it
   // goes through. Since I was sending to myself, it looks like
   // it sent it both directions ...    
   Send a text message via SMS with text:
      cStr = "sms:number:text to send
      
   It should be noted that this API has been deprecated by Google, which means
   it could stop working at any time ...
   
   Apparently you can create a wide range of other chart types using this API
   as well. I haven't experimented.
   -----------------------------------------------------------------------------
*/

// Testing code -- you would want to use your own, or in your application
// access this differently.

// name of the image
cImg = "MyTestQRCode.png"

// build the string to be converted to the QR Code -- this is testing
// the VCARD format see notes in the header comments for more options:
cStr = "BEGIN:VCARD"       // REQUIRED
cStr += "%0A" // new line
cStr += "VERSION:3.0"       // REQUIRED
cStr += "%0A" // new line
cStr += "N:Smith;Fred"      // REQUIRED: lastname semicolon firstname
cStr += "%0A" // new line
cStr += "FN:Fred Smith"     // REQUIRED: Formatted Name
cStr += "%0A" // new line
cStr += "ORG:My Cool Business" // organization or business
cStr += "%0A" // new line
cStr += "TITLE:Salesman"  // title in organization
cStr += "%0A" // new line
cStr += "TEL;TYPE=WORK,VOICE:(999) 555-1212"
cStr += "%0A" // new line
cStr += "TEL;TYPE=CELL:(999) 555-1515"
cStr += "%0A" // new line
cStr += "TEL;TYPE=HOME,VOICE:(999) 555-1213"
cStr += "%0A" // new line
cStr += "ADR;TYPE=WORK:;;1000 Main St;SomeCity;IO;99999;United States of America" // Country = "United States of America"
cStr += "%0A" // new line
cStr += "LABEL;TYPE=WORK:1000 Main St\nSomeCity, IO 99999\nUnited States of America" // \n is new line
cStr += "%0A" // new line
cStr += "ADR;TYPE=HOME:;;2000 Country Lane;SomeCity;IO;99991;United States of America"
cStr += "%0A" // new line
cStr += "LABEL;TYPE=HOME:2000 Country Lane\nSomeCity, IO 99991\nUnited States of America"
cStr += "%0A" // new line
cStr += "EMAIL;TYPE=PREF,INTERNET:fred.smith@MyCoolBusiness.com" // use a real address obviously
cStr += "%0A" // new line
cStr += "URL:http://www.MyCoolBusiness.com" // use a real site obviously
cStr += "%0A" // new line
cStr += "NOTE:Please visit our company website ... you'll like it!" // use a real site obviously
cStr += "%0A" // new line
cStr += "END:VCARD"            // REQUIRED

// size of the QR Code:
nSize = 400

// returns a message -- error or success:
? GenerateQR( cImg, cStr, nSize )

return


function GenerateQR( cImg, cStr, nSize )
   
   //local cImg, cStr, cURL, cPar, objHttp, objStream
   local cURL, cPar, objHttp, objStream
   if pCount() < 3
      return "Problem -- missing parameters -- there are three required!"
   endif
   
   try
      if empty( cImg )
         cImg = "TestQRImg.png"
      endif
      
      if empty( cStr )
         return "Problem -- missing cStr parameter"
      endif
      
      if nSize < 10 or nSize > 520
         return "Size is invalid"
      endif
      
      // Google API Call -- don't change this address:
      cURL = "https://chart.googleapis.com/chart"  // POST do not use ?
      // max size is 520?
      cPar = "chs="+nSize+"x"+nSize+"&cht=qr&chl=" + cStr  // image format 520x520
      objHttp = new OleAutoClient("Msxml2.XMLHTTP")
      objHttp.open("POST",cURL,false)
      objHttp.setRequestHeader("Content-Type", "application/x-www-form-urlencoded")
      objHttp.send(cPar)
      if not empty(objHttp.responseText)
          objStream = new OleAutoClient("ADODB.Stream")
          objStream.Open()
          objStream.Type = 1 // adTypeBinary
          objStream.Write(objHttp.responseBody)
          objStream.SaveToFile(cImg,2) //adSaveCreateOverWrite = 2
          objStream.Close()
      else
       *? "Login Failed! " + datetime()+cImg
      endif

   catch( exception e )
      ? "Error:   " + e.code
      ? "Message: " + e.message
      ? "Line:    " +e.lineNo
      
   endtry
   
   // if here, we have success:
   return "Image: '"+ cImg + "' created ..."
return
