/* 
   INDEXQRY.CC
   9-12.2008
   Revised 10-12-2008, Jan 25, 2009, 31. Jan 2009
   Ivar B. Jessen
   Based on IndexQry.cc by Ken Mayer
   
   A custom query class with a "selIndex()" method attached to the rowset.
   
   Note that this file has two classes: the query (IndexQry)
   and a form (IndexQryForm)

   Usage:
      set procedure to IndexQry.cc additive
      q = new IndexQry()
      q.sql = "select * from 'MyTable'"
      q.active = true
      // so far, just like a normal query, you can also 
      // assign a database before the active statement
      q.selIndex()          // blank will bring up a form
         OR
      q.selIndex("MyIndex") // will attempt to find it,
                            // and if found, will set it ...

   NOTE: This code always sets the row pointer to the top of
   the rowset (this.rowset.first() ) -- there are two
   places it does this -- if you would rather it didn't,
   simply comment them out ...

*/

class IndexQry of QUERY custom

   Procedure selIndex ( cIndexName )
      if empty( this.rowset.tableName )
         msgbox( "Error -- invalid table name" )
         return
      endif

      tDef = new tableDef()
      tDef.database := this.database
      tDef.tableName := this.rowset.tableName
      tDef.load()
      indexFlag = iif( tDef.indexes.size > 0, true, false )

      if indexFlag
         if empty( cIndexName )
            fQry = new IndexQryForm()
            fQry.aIndexes = new Array()
            // Load array used in form for listbox:
            fQry.aIndexes.add( "<No Index>" )
            for i = 1 to tDef.indexes.size
               fQry.aIndexes.Add( tDef.indexes[i].indexName )
            next i

            fQry.aIndexes.sort(2)
            fQry.indexListBox.dataSource := "array fQry.aIndexes"
            fQry.indexListBox.curSel     := 1
            // Open it!
            fQry.ReadModal()
            // set it!
            if fQry.indexListBox.value == "<No Index>"
               this.rowset.indexName := ""
            else
               this.rowset.indexName := fQry.indexListBox.value
            endif
            this.rowset.first()

         else
            // let's try to find it ...
            this.rowset.indexName := ""
            for i = 1 to tDef.indexes.size
               if upper(cIndexName) == tDef.indexes[i].indexName
                  this.rowset.indexName := cIndexName
                  this.rowset.first()
                  exit
               endif
            next i
            // if the indexName is still empty:
            if empty( this.rowset.indexName )
                  msgbox( "Could not find indexName: " + cIndexName + ".")
               endif
         endif // empty( cIndexName )
      else
         // Let user know somewhere there's a mistake ... <G>
         msgbox( "There are no indexes for this table" )
      endif
  
endclass

// end of class: INDEXQRY

// form used above in SelIndex routine:

class IndexQryForm of FORM
   with (this)
      scaleFontBold = false
      height = 11
      left = 31.2857
      top = 4.9545
      width = 40
      text = "Select index for data"
      clientEdge = true
      topMost = true
      autoCenter = true
      mdi = false
      smallTitle = true
      sizeable = false
      escExit = false
      maximize = false
      minimize = false
   endwith


   this.INDEXLISTBOX = new LISTBOX(this)
   with (this.INDEXLISTBOX)
      height = 5.7727
      left = 1.8571
      top = 0.8182
      width = 35.5714
      id = 101
      colorHighLight = "white/blue"
   endwith


   this.TEXTEXPL = new TEXT(this)
   with (this.TEXTEXPL)
      height = 2.1818
      left = 2.5714
      top = 6.9545
      width = 34.1429
      border = true
      fontSize = 8
      fontItalic = true
      text = "Select the index you wish to use. " + ;
      		 "When you click the button below, " + ;
      		 "whatever index you have chosen will take effect on your table."
      borderStyle = 10	// Etched Out
   endwith


   this.DONEPUSHBUTTON = new PUSHBUTTON(this)
   with (this.DONEPUSHBUTTON)
      height = 1.0909
      left = 17.2857
      top = 9.4091
      width = 4.7143
      text = ""
      default = true
      upBitmap = "RESOURCE #1005"
      downBitmap = "RESOURCE #1006"
      speedTip = "Done"
      value = false
      onClick = {; form.close() }
   endwith

endclass

// end of file: Indexqry.cc



