/*
   Class PixCalendar   A custom dBASE Calendar Component Class
   Purpose: Get and set dates on visual calendar control
   Author: A. A. Katz
   Date: 03-18-1997, 04-01-1997
   Copyright: A. A. Katz. All rights reserved. However, this class
      may be used by developers in their applications. They
      may modify source code. They may NOT resell this
      custom class as a component or class.
   Usage: Register this class in dBASE File/Set up Custom Controls
          Drop on form as needed
          To use from dUFLP:
             set procedure to :dUFLP:PixCalendar.cc additive

   Main Methods: setDate(dDate) sets new calendar default date
                 GetDate()  returns the selected date on the calendar
   Main Property: Date (this.date). Contains the currently selected date.
                              If no date is sent on instantiation,
                              today's date is used
   Upgraded  01/28/2000    AAK  For Improved interprocess messaging
   Added Support for an "owner object" with callback if
        it has a setDate method
   Support for automatic field updates if this.currentfield
        object has a date-type value property
   Fixed  11/12/2002  Uses date class for generating date values
        (instead of ctod).  Now works regardless of MDY,YMD, or DMY setting.
   Metric changed to Pixel by Jean-Pierre Martel, 2004-07-17
   French translation by Jean-Pierre Martel, 2004-07-17
   German translation by Jean-Pierre Martel, 2004-07-17
   Spanish translation by Jean-Pierre Martel, 2004-07-17
*/

Class PixCalendar(oF) of Container(oF) Custom
   with (this)
      onOpen = class::calendar_OnOpen
      onDesignOpen = {; this.parent.metric=6 }
      left = 1
      top = 2
      width = 153
      height = 147
      colorNormal = "BtnFace"
      OnDateClick = null // takes a function pointer,
                         // fires when date button clicked
   endwith

   this.aCurrentOwners = new array() // init exclusive owner
   this.aCurrentFields = new array() // update any number of fields
   this.language       = "English"   // or "French" or "German" or "Spanish"

   this.TEXT2 = new TEXT(this)
   with (this.TEXT2)
      alignVertical = 1 // Middle
      height = 14
      left = 7
      top = 26
      width = 133
      fontName = 'Arial'
      fontSize = 7
      text = "  S    M    T    W     T    F    S"
      colorNormal = "white/0x804000"
      pageno = 0
   endwith

   this.PUSHBUTTON1 = new CalButton(this)
   with (this.PUSHBUTTON1)
      left = 6
      top = 40
   endwith

   this.PUSHBUTTON2 = new CalButton(this)
   with (this.PUSHBUTTON2)
      left = 25
      top = 40
   endwith

   this.PUSHBUTTON3 = new CalButton(this)
   with (this.PUSHBUTTON3)
      left = 44
      top = 40
   endwith

   this.PUSHBUTTON4 = new CalButton(this)
   with (this.PUSHBUTTON4)
      left = 63
      top = 40
   endwith

   this.PUSHBUTTON5 = new CalButton(this)
   with (this.PUSHBUTTON5)
      left = 82
      top = 40
   endwith

   this.PUSHBUTTON6 = new CalButton(this)
   with (this.PUSHBUTTON6)
      left = 101
      top = 40
   endwith

   this.PUSHBUTTON7 = new CalButton(this)
   with (this.PUSHBUTTON7)
      left = 120
      top = 40
   endwith

   this.PUSHBUTTON8 = new CalButton(this)
   with (this.PUSHBUTTON8)
      left = 6
      top = 55
   endwith

   this.PUSHBUTTON9 = new CalButton(this)
   with (this.PUSHBUTTON9)
      left = 25
      top = 55
   endwith

   this.PUSHBUTTON10 = new CalButton(this)
   with (this.PUSHBUTTON10)
      left = 44
      top = 55
   endwith

   this.PUSHBUTTON11 = new CalButton(this)
   with (this.PUSHBUTTON11)
      left = 63
      top = 55
   endwith

   this.PUSHBUTTON12 = new CalButton(this)
   with (this.PUSHBUTTON12)
      left = 82
      top = 55
   endwith

   this.PUSHBUTTON13 = new CalButton(this)
   with (this.PUSHBUTTON13)
      left = 101
      top = 55
   endwith

   this.PUSHBUTTON14 = new CalButton(this)
   with (this.PUSHBUTTON14)
      left = 120
      top = 55
   endwith

   this.PUSHBUTTON15 = new CalButton(this)
   with (this.PUSHBUTTON15)
      left = 6
      top = 70
   endwith

   this.PUSHBUTTON16 = new CalButton(this)
   with (this.PUSHBUTTON16)
      left = 25
      top = 70
   endwith

   this.PUSHBUTTON17 = new CalButton(this)
   with (this.PUSHBUTTON17)
      left = 44
      top = 70
   endwith

   this.PUSHBUTTON18 = new CalButton(this)
   with (this.PUSHBUTTON18)
      left = 63
      top = 70
   endwith

   this.PUSHBUTTON19 = new CalButton(this)
   with (this.PUSHBUTTON19)
      left = 82
      top = 70
   endwith

   this.PUSHBUTTON20 = new CalButton(this)
   with (this.PUSHBUTTON20)
      left = 101
      top = 70
   endwith

   this.PUSHBUTTON21 = new CalButton(this)
   with (this.PUSHBUTTON21)
      left = 120
      top = 70
   endwith

   this.PUSHBUTTON22 = new CalButton(this)
   with (this.PUSHBUTTON22)
      left = 6
      top = 85
   endwith

   this.PUSHBUTTON23 = new CalButton(this)
   with (this.PUSHBUTTON23)
      left = 25
      top = 85
   endwith

   this.PUSHBUTTON24 = new CalButton(this)
   with (this.PUSHBUTTON24)
      left = 44
      top = 85
   endwith

   this.PUSHBUTTON25 = new CalButton(this)
   with (this.PUSHBUTTON25)
      left = 63
      top = 85
   endwith

   this.PUSHBUTTON26 = new CalButton(this)
   with (this.PUSHBUTTON26)
      left = 82
      top = 85
   endwith

   this.PUSHBUTTON27 = new CalButton(this)
   with (this.PUSHBUTTON27)
      left = 101
      top = 85
   endwith

   this.PUSHBUTTON28 = new CalButton(this)
   with (this.PUSHBUTTON28)
      left = 120
      top = 85
   endwith

   this.PUSHBUTTON29 = new CalButton(this)
   with (this.PUSHBUTTON29)
      left = 6
      top = 100
   endwith

   this.PUSHBUTTON30 = new CalButton(this)
   with (this.PUSHBUTTON30)
      left = 25
      top = 100
   endwith

   this.PUSHBUTTON31 = new CalButton(this)
   with (this.PUSHBUTTON31)
      left = 44
      top = 100
   endwith

   this.PUSHBUTTON32 = new CalButton(this)
   with (this.PUSHBUTTON32)
      left = 63
      top = 100
   endwith

   this.PUSHBUTTON33 = new CalButton(this)
   with (this.PUSHBUTTON33)
      left = 82
      top = 100
   endwith

   this.PUSHBUTTON34 = new CalButton(this)
   with (this.PUSHBUTTON34)
      left = 101
      top = 100
   endwith

   this.PUSHBUTTON35 = new CalButton(this)
   with (this.PUSHBUTTON35)
      left = 120
      top = 100
   endwith

   this.PUSHBUTTON36 = new CalButton(this)
   with (this.PUSHBUTTON36)
      left = 6
      top = 115
   endwith

   this.PUSHBUTTON37 = new CalButton(this)
   with (this.PUSHBUTTON37)
      left = 25
      top = 115
   endwith

   this.MONTHDISPLAY = new TEXT(this)
   with (this.MONTHDISPLAY)
      height = 17
      left = 23
      top = 6
      width = 100
      border = true
      colorNormal = "white/0x804000"
      alignHorizontal = 1 // Center
      fontSize = 8
      fontBold = true
      text = "January"
      borderStyle = 4 // Single
      pageno = 0
   endwith

   this.YEARDISPLAY = new TEXT(this)
   with (this.YEARDISPLAY)
      height = 17
      left = 79
      top = 116
      width = 45
      colorNormal = "white/0x804000"
      alignHorizontal = 1 // Center
      fontSize = 8
      text = "2004"
      fontBold = true
      borderStyle = 4 // Single
      pageno = 0
   endwith

   this.MONTHUPBUTTON = new PUSHBUTTON(this)
   with (this.MONTHUPBUTTON)
      onClick = class::MONTHUPBUTTON_ONCLICK
      height = 18
      left = 124
      width = 16
      top = 6
      text = ""
      upBitmap = "RESOURCE #854"
      downBitmap = "RESOURCE #854"
      borderStyle = 2 // Lowered
      colorNormal = "0x804000/BtnFace"
      pageno = 1
      value = false
      speedbar = true
   endwith

   this.MONTHDOWNBUTTON = new PUSHBUTTON(this)
   with (this.MONTHDOWNBUTTON)
      onClick = class::MONTHDOWNBUTTON_ONCLICK
      height = 18
      left = 7
      width = 16
      text = ""
      upBitmap = "RESOURCE #853"
      downBitmap = "RESOURCE #853"
      borderStyle = 2 // Lowered
      top = 6
      colorNormal = "0x804000/BtnFace"
      pageno = 1
      value = false
      speedbar = true
   endwith

   this.YEARUPBUTTON = new PUSHBUTTON(this)
   with (this.YEARUPBUTTON)
      onClick = class::YEARUPBUTTON_ONCLICK
      height = 18
      left = 125
      top = 116
      width = 16
      text = ""
      upBitmap = "RESOURCE #854"
      downBitmap = "RESOURCE #854"
      borderStyle = 2 // Lowered
      colorNormal = "0x804000/BtnFace"
      pageno = 1
      speedbar = true
      value = false
   endwith

   this.YEARDOWNBUTTON = new PUSHBUTTON(this)
   with (this.YEARDOWNBUTTON)
      onClick = class::YEARDOWNBUTTON_ONCLICK
      height = 18
      left = 63
      width = 16
      top = 116
      text = ""
      upBitmap = "RESOURCE #853"
      downBitmap = "RESOURCE #853"
      borderStyle = 2 // Lowered
      colorNormal = "0x804000/BtnFace"
      pageno = 1
      value = false
      speedbar = true
   endwith

   Function setDate(dDate,cRepaint)
      /*
      Purpose: Only entry point to set default/current selected
      Date. Designed for Internal/External calls
      */

      // set defaults for missing params
      dDate      = iif(empty(dDate),date(),dDate)
      cRepaint   = iif(empty(cRepaint),'Y',cRepaint)
      this.Today = Date()   // Today's date
      this.date  = dDate    // Selected date
      /*
      this.CalDate  = ctod( str(month(this.date),2)+;
         '/01/'+str(year(this.date),4))  // Used internally to
                                          // navigate calendar
      fixed by Rick as follows
      */
      this.CalDate  = new Date( year(this.date), month(this.date)-1, 1 )
     // Used internally to navigate calendar
      if cRepaint = 'Y'
         this.DrawMonth()                 // Draw calendar
      endif
      return

   Function GetDate()
      // Purpose: returns calendar's selected date
      return this.date

   Function calendar_OnOpen
      // Container's OnOpen
      // Purpose: Ensures default date is set
      #if __version__ > 2.5
         this.MonthDownButton.systemTheme := false
         this.MonthUpButton.systemTheme   := false
         this.YearDownButton.systemTheme  := false
         this.YearUpButton.systemTheme    := false
      #endif
      if this.language == "French"
         this.Text2.text = "  D    L    M    M     J    V    S"
         this.MonthDisplay.text = "Janvier"
      elseif this.language = "German"
         this.Text2.text = "  S    M    D    M     D    F    S"
         this.MonthDisplay.text = "Januar"
      elseif this.language = "Spanish"
         this.Text2.text = "  S    M    D    M     D    F    S"
         this.MonthDisplay.text = "Enero"
      elseif this.language = "Italian"
         this.Text2.text = "  D    L    M    M     G    V    S"
         this.MonthDisplay.text = "Gennaio"
      endif
      if type('this.CalDate') # 'D'
         this.CalDate = new date()
      endif
      this.setDate(this.date)
      * this.setDate(Date())  // JPM
   return

   Function drawMonth
      /*
      Purpose: Draws the selected month on the calendar.
      Param: this = reference to container. Make sure that
                    all calls to this method or to setDate() are
                    scoped to the container
      */

      local nMonth
      nMonth = Month(this.CalDate)
      if this.language     == "English"
         this.monthdisplay.text = cMonth(this.CalDate)
      elseif this.language == "French"
         this.monthdisplay.text = this.Translate_to_French(nMonth)
      elseif this.language == "German"
         this.monthdisplay.text = this.Translate_to_German(nMonth)
      elseif this.language == "Spanish"
         this.monthdisplay.text = this.Translate_to_Spanish(nMonth)
      elseif this.language == "Italian"
         this.monthdisplay.text = this.Translate_to_Italian(nMonth)
      endif
      this.YearDisplay.text  =  str(year(this.CalDate),4)

      // Get the last day of the month
      cMonth = str(month(this.CalDate),2)
      if at(cMonth,' 9 4 611') > 0       // if 30 day month
         nLastDay = 30
      elseif cMonth = ' 2'               // if February, do leap year
         nLastDay = iif(this.CalDate.Year%4 = 0, 29,28)
      else
         nLastDay = 31                  // Otherwise, 31
      endif
      for n = 1 to dow(this.CalDate)    // draw empty starting days
         cN        = trim(ltrim(str(n,2)))
         cColor    = 'this.pushbutton'  +cN +'.Colornormal'
         &cColor   = 'black/silver'
         cText     = 'this.pushbutton'  +cN +'.text'
         &cText    = ''
         cEnabled  = ' this.pushbutton' +cN +'.enabled'
         &cEnabled = false
      next
      nDay = 1
      for n = dow(this.CalDate) to 37    // draw days of the month
         cN           = trim(ltrim(str(n,2)))
         cColor       = 'this.pushbutton'  +cN +'.Colornormal'
         &cColor.     = 'black/silver'
         cText        = 'this.pushbutton'  +cN +'.text'
         cEnabled     = ' this.pushbutton' +cN +'.enabled'
         if nDay > nLastDay
           &cText.    = ' '
           &cEnabled. = false
         else
           &cText.    = str(nDay,2)
           &cEnabled. = true
         endif
         nDay ++
      endfor

      class::highlightToday()
      // If selected date falls on this calendar page
      // change color and redisplay

      if month(this.CalDate) = month(this.date) and ;
         year(this.CalDate)  = year(this.Date)
         n = dow(this.CalDate) + day(this.Date) -1
         cButton             = 'this.pushbutton'+ltrim(str(n,2))+'.colornormal'
         &cButton            = 'W+/0x804000'
      endif
   return

   Function Translate_to_French(nMonth)
      local c
      do case
         case nMonth = 1
            c = "Janvier"
         case nMonth = 2
            c = "Fvrier"
         case nMonth = 3
            c = "Mars"
         case nMonth = 4
            c = "Avril"
         case nMonth = 5
            c = "Mai"
         case nMonth = 6
            c = "Juin"
         case nMonth = 7
            c = "Juillet"
         case nMonth = 8
            c = "Aot"
         case nMonth = 9
            c = "Septembre"
         case nMonth = 10
            c = "Octobre"
         case nMonth = 11
            c = "Novembre"
         case nMonth = 12
            c = "Dcembre"
       endcase
       return(c)

   Function Translate_to_German(nMonth)
      local c
      do case
         case nMonth = 1
            c = "Januar"
         case nMonth = 2
            c = "Februar"
         case nMonth = 3
            c = "Mrz"
         case nMonth = 4
            c = "April"
         case nMonth = 5
            c = "Mai"
         case nMonth = 6
            c = "Juni"
         case nMonth = 7
            c = "Juli"
         case nMonth = 8
            c = "August"
         case nMonth = 9
            c = "September"
         case nMonth = 10
            c = "Oktober"
         case nMonth = 11
            c = "November"
         case nMonth = 12
            c = "Dezember"
       endcase
       return(c)

   Function Translate_to_Spanish(nMonth)
      local c
      do case
         case nMonth = 1
            c = "Enero"
         case nMonth = 2
            c = "Febrero"
         case nMonth = 3
            c = "Marzo"
         case nMonth = 4
            c = "Abril"
         case nMonth = 5
            c = "Mayo"
         case nMonth = 6
            c = "Junio"
         case nMonth = 7
            c = "Julio"
         case nMonth = 8
            c = "Agosto"
         case nMonth = 9
            c = "Setiembre"
         case nMonth = 10
            c = "Octubre"
         case nMonth = 11
            c = "Noviembre"
         case nMonth = 12
            c = "Diciembre"
       endcase
       return(c)

   Function Translate_to_Italian(nMonth)
      local c
      do case
         case nMonth = 1
            c = "Gennaio"
         case nMonth = 2
            c = "Febbraio"
         case nMonth = 3
            c = "Marzo"
         case nMonth = 4
            c = "Aprile"
         case nMonth = 5
            c = "Maggio"
         case nMonth = 6
            c = "Giugno"
         case nMonth = 7
            c = "Luglio"
         case nMonth = 8
            c = "Agosto"
         case nMonth = 9
            c = "Settembre"
         case nMonth = 10
            c = "Ottobre"
         case nMonth = 11
            c = "Novembre"
         case nMonth = 12
            c = "Dicembre"
       endcase
       return(c)

   Function monthUpButton_onClick
      // Increments calendar months
      nNewMonth = month(this.parent.CalDate)
      nNewMonth ++
      if nNewMonth > 12
         nNewMonth = 1
         nNewYear  = year(this.parent.CalDate)+1
      else
         nNewYear  = year(this.parent.CalDate)
      endif
      this.parent.date    = new Date( nNewYear, nNewMonth-1, day(this.parent.date) )
      this.parent.CalDate = new Date( nNewYear, nNewMonth-1, 1 )
      this.parent.drawmonth()
      return

   Function monthDownButton_onClick
      // Decrements calendar months
      nNewMonth = month(this.parent.CalDate)
      nNewMonth --
      if nNewMonth < 1
          nNewMonth = 12
          nNewYear = year(this.parent.CalDate)-1
      else
          nNewYear = year(this.parent.CalDate)
      endif
      this.parent.date    = new Date( nNewYear, nNewMonth-1, day(this.parent.date) )
      this.parent.CalDate = new Date( nNewYear, nNewMonth-1, 1 )
      this.parent.drawmonth()
      return

   Function yearUpBUTTON_onClick
      // Increments calendar years
      dDate = this.parent.Date
      this.parent.date    =  new Date( year(dDate)+1,  month(dDate)-1, day(this.Parent.Date) )
      this.parent.calDate = new Date( year(dDate)+1,  month(dDate)-1, 1 )
      this.parent.drawmonth()
      return

   Function yearDownButton_OnClick
      // Decrements calendar Year
      dDate = this.parent.Date
      this.parent.date    = new Date( year(dDate)-1,  month(dDate)-1, day(this.Parent.Date) )
      this.parent.calDate = new Date( year(dDate)-1,  month(dDate)-1, 1 )
      this.parent.drawmonth()
      return

   Function highlightToday()
      // Marks "today" button in red/silver
      if month(this.Date) = month(date()) and ;
         year(this.Date)  = year(date())
         n = dow(this.CalDate)+day(Date())-1
         cButton          = 'this.pushbutton'+ltrim(str(n,2))+'.colornormal'
         &cButton         = "white/red" // "'Red/Silver'
      endif
      return

   Function RegisterOwners(oOwner,bExclusiveByClass)
      /*
      Register external owner objects - called externally
      bExclusive kills off all other owners but this one
      bExclusive By Class means that only One pushbutton or
      one container of the same class may be registered at a time
      */
      private cClassName  // init var for exclusive
      bExclusiveByClass = iif(empty(bExclusiveByClass),false,bExclusiveByClass)
      // Go through array over and over until you remove all
      // of a matching class to the one passed in oOwnerRef
      if bExclusiveByClass
         for n = 1 to this.aCurrentOwners.size
            if type('this.aCurrentOwners[n]') = 'O' and ;
               this.aCurrentOwners[n].className = oOwner.ClassName
               this.aCurrentOwners.delete(n)
            endif
         next
      endif
      // Add new owner
      this.aCurrentOwners.add(oOwner)
      return true

   Function RegisterFields(oFieldRef)
      // Register external owner objects - called externally
      this.aCurrentFields.add(oFieldRef)
      return true

   Function unRegister(oObj)
      // Purpose: Removes all external references
      for n = 1 to this.aCurrentOwners.size
         if empty(oObj) or (this.aCurrentOwners[n] = oObj)
            this.aCurrentOwners[n] = null
         endif
      next
      for n = 1 to this.aCurrentFields.size
         if empty(oObj) or (this.aCurrentOwners[n] = oObj)
            this.aCurrentFields[n] = null
         endif
      next
      return

   Function enable(bEnable)
     // Purpose: turns pusbuttons on or off
     // Call from container only
      oFirst = this.first
      oCurrent = oFirst
      do
         ////// see if it has an enabled property
         if upper(oCurrent.className) = 'CALBUTTON'
            oCurrent.enabled = bEnable
         endif
         oCurrent = oCurrent.before
      until oCurrent = oFirst
      return true

endclass  // end of class::PixCalendar

Class CalButton(formObj) of Pushbutton(formObj)// Custom
// Calbutton: Custom pushbutton class
// Purpose: contains method that fires when date is selected
   #if __version__ > 2.5
      // for these toolbuttons, don't use the system theme ...
      this.systemTheme := false // for dBASE Plus 2.6 or later
   #endif

   with (this)
      height = 17
      left = 25
      onclick = class::OnClick
      top = 130
      width = 21
      text = " "
      fontSize = 8
      pageno = 1
      speedbar = true
      borderStyle = 2 // Lowered
   endwith

   Function OnClick
      // Purpose: Resets date when button pushed
      if empty(this.text)
         return null
      endif
      // build a date from this button text and
      // current month/year as displayed
      cDispMonth = this.parent.monthdisplay.text
      nPos = 0
      if this.parent.language == "English"
         do
            nPos++
         until cDispMonth = cMonth( new Date( year(date()), nPos-1, 1) )
      elseif this.parent.language == "French"
         do
            nPos++
         until cDispMonth = this.parent.Translate_to_French(Month( new Date( year(date()), nPos-1, 1) ))
      elseif this.parent.language == "German"
         do
            nPos++
         until cDispMonth = this.parent.Translate_to_German(Month( new Date( year(date()), nPos-1, 1) ))
      elseif this.parent.language == "Spanish"
         do
            nPos++
         until cDispMonth = this.parent.Translate_to_Spanish(Month( new Date( year(date()), nPos-1, 1) ))
      elseif this.parent.language == "Italian"
         do
            nPos++
         until cDispMonth = this.parent.Translate_to_Italian(Month( new Date( year(date()), nPos-1, 1) ))
      endif
      // turn off this color if it shows onscreen
      this.parent.setDate( ( new Date( val(this.parent.yeardisplay.text),;
                           nPos-1, val(this.text))), 'N' )
      // check all the buttons to see if any are highlighted as
      // "selected" date. If so, ditch the highlight
      oStart = this.parent.pushbutton1
      oCurrent = oStart
      do
         if upper(oCurrent.classname) = 'CALBUTTON'
            if oCurrent.colornormal = 'W+/0x804000'
               oCurrent.colornormal = 'black/silver'
            endif
         endif
         oCurrent = oCurrent.before
      until oCurrent = oStart
      // highlight "today" if appropriate
      this.parent.highlightToday()
      // highlight this button as current date
      this.colornormal = 'W+/0x804000'
      /*
      This class now supports the following:
      1) a passed function pointer for callback
      2) a Current "owner" object for callback
      3) a Current "field" for auto update
      */
      // current owners callback
      // requires that it also have a "setdate" method
      private n
      for n =1 to this.parent.aCurrentOwners.size
         if type('this.parent.aCurrentOwners[n]') = 'O' and ;
            type('this.parent.aCurrentOwners[n].setDate') = 'FP' // if setdate exists
            this.parent.aCurrentOwners[n].setDate(this.parent.Date)
         endif
      next
      ///// current field
      for n = 1 to this.parent.aCurrentFields.size
         if type('this.parent.aCurrentFields[n].value') = 'D' // if it's a date field
            this.parent.aCurrentFields[n].value = this.parent.Date
         endif
      next
      return null

Endclass
