/*
  ----------------------------------------------------------------
   MISCAPI.PRG

   This program will contain some API routines as written by
   various folk that appears to be useful. I will add whatever
   commentary to help in using it as I stumble across it ...

   Much of this code you will want to copy and paste elsewhere
   for use in your own applications ...

   Original: October 16, 1999

   Included here:
   
   BM_OS() -- returns information about the OS, including whether
              it is 32-bit or 64-bit.

   Capslock() -- toggles the Capslock key on/off

   ConnectNetDrive() -- An API connection to a network drive,
                        includes optional password.

   FindChildByClass() -- returns the windows handle
                         of a control on a form

   FindChildByText() --  returns the windows handle of
                         a child form based on text

   FindVdBWin() -- returns the windows handle of a 
                   Visual dBASE window 

   GetCD() -- returns drive letter for CD ...

   GetImageName() -- returns the dataSource of an
                     image that has been dragged and
                     dropped into the ImageViewer in
                     Visual dBASE.

   getInternetConnection() -- returns true/false if there is or
       is not an internet connection for the computer it is run on.

   GetLabel() -- returns disk label (if any)

   GetPosition() -- returns mouse position *anywhere* on the
      screen.

   GetPositionInForm() -- similar to above but must be 
      called from a form, and returns position of mouse
      relative to form.

   GetSerialNumber() -- Returns the serial number on the disk
      drive. 

   GetSystemName() -- API call to return the Computer Name

   GetWindowsFolders() -- Returns a list of the 'special folders'
                          used by Windows and the paths to them,
                          in an associative array.

   GetWindowsCSIDLPath() -- Returns the path to a specific
                            folder as used by Windows Vista ... allows
                            the folder to be created if it doesn't exist.

   HiddenDos() -- Run a dos application "hidden" -- so
     DOS window does not appear ...

   IsCBDropped() -- returns true/false if combobox has been
       "dropped"

   IsReadOnly() -- returns true/false if a file's readonly
      attribute is set to true.

   IsRunning() -- returns true/false if an external application is
      currently running.

   IsURLGood() -- returns true/false if a URL passed to the
      function is good. Useful before calling, say, "OpenURL()" 
      (see below). 

   MakeDirTree() -- will create a directory tree if
      needed, using a couple of files in the Windows
      system folder to do so.

   NoSysClose() -- Turns off system close menu on form or application

   NoSysCloseBtn() -- similar to above, handles 'x' button

   OpenURL() -- a small routine to open a URL with your
      "default" browser. Might be useful for a help system.

   PrintHTM() -- Will take a tempfile and print the HTML ...

   Play_midi() -- plays a MIDI file ...

   PrntScrn() -- a small routine to call the Alt+PrintScrn
      keystroke -- uses a parameter to determine whether
      to grab the full screen or the active window ...

   Reboot() -- used to reboot a computer ...

   RD() -- removes a directory

   RenameDirectory() -- Renames a directory

   RunHidden() -- function to run a DOS command or string of
      commands in a hidden DOS window.

   RunIt() -- a function that will run an application (windows app)
      from within your own application in whatever state
      you wish (Normal, Minimized, Maximized)

   Run_AVI() -- runs an AVI file ...

   ScrollHome() -- moves the screen positioning for
      screens (such as a reportViewer) to the "home"
      (top/left) position.

   SendMail() -- a quick routine to send email to 
      a standard EMail address. Note that you must be
      connected to the internet at the time that you issue
      the call to this routine.

   SetLabel() -- Sets the disk label

   SetReadOnly() -- Sets read-only attribute of file
      either on or off.

   ShowInspector() -- routine adapted from Jim Sare's FindApp
      routine. Brings inspector to the upper left corner if it
      is hidden in some "negative" location.

   SizeEx() -- returns the file size of a file, even if it is
      larger than 2gb in size. dBASE has a problem with files
      that are that large and the file object will return
      an invalid size ...

   SmallFonts() -- determines if a user has "smallfonts" set
      as their setting for their application.

   sysClose() -- similar to "NoSysClose" and "NoSysCloseBtn"
      above but takes parameter to turn this on and off ...

   SysDir() -- returns the Windows System path.

   SystemFonts() -- returns the system font setting, for use
      in determining sizing of controls for an application.

   TableOpenCount() -- uses BDE API to determine number of
      times a table is open.

   WindowsDirectory() -- returns location Windows is installed
      in ...

   WindowExists() -- returns logical if a Window exists with
      a specific name.

   ---------------------------------------------------------------- 
*/

msgbox( "This program is not meant to be executed, "+;
        "but contains useful API calls that can be copied "+;
        "and used in your own applications.","Can't run this!", 16 )
return


FUNCTION bm_os()                            // version of Windows
   /*
     ------------------------------------------------------------------ 
      BM_OS()
      Posted by (unknown), dBASE newsgroups
         If it helps, the original documentation was in French.
      Reposted by Mervyn Bick
      July, 2016

      Returns information about Windows -- 64-bit versus 32-bit,
              if 64-bit it returns more detail, such as which
              version of Windows ...

      Usage: ? BM_OS()
     ------------------------------------------------------------------ 
  */
   LOCAL oLocator                           // OLEAutoclient Object
   LOCAL oSys                               // Operating System Object
   LOCAL oWMI                               // ConnectServer Object
   LOCAL cReturn                            // Return from Function
   
   try
       oLocator = new OleAutoClient( "WbemScripting.SWbemLocator" )
       oWMI = oLocator.ConnectServer( ".", "root\cimv2" )
       oSys = oWMI.ExecQuery( 'Select * from Win32_OperatingSystem' )
       cReturn = trim(oSys[ 0 ].Caption) + ", " + oSys[ 0 ].OSArchitecture
   catch(exception e)
       cReturn = trim(oSys[ 0 ].Caption) + ", 32-bit"
   endtry

   // clear out memory
   oLocator := null
   oSys     := null
   oWMI     := null
RETURN cReturn 

function Capslock
  /*
     ------------------------------------------------------------------ 
      Capslock
      Posted by Romain Strieff, dB2K Programming Newsgroup
      March 1, 2002

      Toggles the state of the caps-lock key on 
      the keyboard.

      Usage: Capslock( [<lOnOff>])
             Optional parameter lOnOff
                      logical value: true = on
                                     false = off
             If no parameter is given, this returns the
                current setting of the capslock key.
             If parameter is given, it sets the capslock
                key to on or off, and returns the previous
                state.
     ------------------------------------------------------------------ 
  */
   parameters lOnOff
   #define KEYEVENTF_EXTENDEDKEY 0x0001
   #define KEYEVENTF_KEYUP       0x0002
   #define VK_CAPITAL HToI("14")

   if type("lGetKeyState") # "FP"
      extern cLogical lGetKeyState(cint) user32;
             from "GetKeyState"
   endif
   if type("keybd_event") # "FP"
      extern cVOID keybd_event(cint, cint, cWORD, cWORD) user32
   endif

   if lGetKeyState(VK_CAPITAL) # lOnOff
      //Simulate Key Press
      keybd_event(VK_CAPITAL,;
         HTOI("45"),KEYEVENTF_EXTENDEDKEY,0)
      //Simulate Key Release
      keybd_event(VK_CAPITAL,;
         HTOI("45"),KEYEVENTF_EXTENDEDKEY+KEYEVENTF_KEYUP,0)
   endif

return lGetKeyState(VK_CAPITAL)

function ConnectNetDrive( cDrive, cShare, cPassword )
   /*
       Programmer: Rich Muller
       Date......: April, 2007
       Notes.....: Sets up a network drive as a shared
                   drive, with an optional password if
                   required.
       Usage.....: ConnectNetDrive( "\\someserver\somedrive", "F:", "MyPass" )
   */
   if type( "cPassword" ) = "U"
      cPassword = ""
   endif
   if type( "WNetAddConnection" ) # "FP"
      extern cInt WNetAddConnection( cString, cString, cString ) MPR.DLL;
             FROM "WNetAddConnectionA"
   endif
return WNetAddConnection( cShare, cPassword, cDrive ) == 0

function ConvertLocalToUNC
   parameters cPath
   /* 
      Programmer: Rich Muller
      Date......: August, 2004
      Notes.....: Rich@autotraker.com www.autotraker.com
                  check to see if this path (cPath) is a 
                  drive mapping and convert it
                  to a UNC if it is. If the path is not a 
                  UNC, we return just the path
   */
   if not left(cPath,2)=="\\" // not a UNC
      lpLocalName = left(cPath,2)
      if type( 'WNetGetConnection' ) # 'FP'
         extern culong WNetGetConnection(cString,cString,cptr culong);
             MPR.DLL from "WNetGetConnectionA"
      endif
      cBuff = space( 255 )
      if WNetGetConnection(lpLocalName,cBuff,len(cBuff)) == 0
         // it's a valid share, so build a UNC
         cPath = trim(cBuff) + subStr(cPath,3)
      endif
   endif
return cPath

/* ----------------------------------------------------------------
   Check to see if another Windows Program has terminated before
   continuing in your own code.

   "I wouldn't recommend this approach to prevent multiple
   instances of the same VdB app." -- Jim (instead, see the
   file: AppFlag.CC in the dUFLP library -- KM)

   Posted by Jim Sare in the Borland VdBASE Newsgroups

   Usage:
      copy this code to a startup routine, and set the text
      as shown below "The Other App's Window Text" ...
   ---------------------------------------------------------------- */
   // Launch other application
   if type( "FindWindowByText" ) # "FP"
      extern CHANDLE FindWindowByText( CLONG, CSTRING ) user32;
             from "FindWindowA"
   endif
   do while true
      sleep 2
      // note: make sure text is accurate
      if FindWindowByText( 0, "The Other App's Window Text" ) = 0
         exit
      endif
   enddo
   // Other application should be done when you fall out of loop
   // here ...


/*
    FindChildByClass -- finds a child control in
    a form by the className
   By Jim Sare
   Posted by Romain Strieff in the dBASE 2000 newsgroups

   Returns the window handle for the control

   Usage:
      FindChildByClass( oObj.hwnd, className )

*/
FUNCTION FindChildByClass(nHandle, cClassName)

	LOCAL nThehWND, xBUFF, nLen, cText, nInd

	If Type("GetClassName") # "FP"
		extern CINT GetClassName;
      (CHANDLE, CPTR, CINT) User32;
				From "GetClassNameA"
	EndIf
	If Type("GetWindow") # "FP"
		extern CHANDLE GetWindow;
      (CHANDLE, CUINT) User32
	EndIf
	#ifndef GW_HWNDNEXT
		#define GW_HWNDNEXT	2
	#endif
	#ifndef GW_CHILD
		#define GW_CHILD	5
	#endif

	nThehWND = GetWindow(nHandle, GW_CHILD)	
	Do While nThehWND # 0							
		xBUFF = Space(41)
		nLen = GetClassName(nThehWND, xBUFF, 80)
		cText = ""
		For nInd = 0 To nLen - 1
			cText += Chr(xBUFF.GetByte(nInd))
		EndFor
		If Upper(cClassName) $ Upper(OEM(cText))
			Exit
		EndIf
		nThehWND = GetWindow(nThehWND, GW_HWNDNEXT)
	EndDo
RETURN nThehWND									
// EOP: FindChildByClass

/*
   FindChildByText -- finds a child window by the
   text in the title bar ...
   By Jim Sare
   Posted by Romain Strieff in the dBASE 2000 newsgroups

   Returns the window handle
*/
FUNCTION FindChildByText(nHandle, cWinText)

	LOCAL nThehWND, xBUFF, nLen, cText, nInd

	If Type("GetWindowText") # "FP"
		extern CINT GetWindowText;
      (CHANDLE, CPTR, CINT) User32;
				From "GetWindowTextA"
	EndIf
	If Type("GetWindow") # "FP"
		extern CHANDLE GetWindow(CHANDLE, CUINT) User32
	EndIf

	#ifndef GW_HWNDNEXT
	   #define GW_HWNDNEXT			2
	#endif
   #ifndef GW_CHILD
      #define GW_CHILD				5
   #endif

   nThehWND = GetWindow(nHandle, GW_CHILD)
	Do While nThehWND # 0						
		xBUFF = Space(41)
		nLen = GetWindowText(nThehWND, xBUFF, 80)
		cText = ""
		For nInd = 0 To nLen - 1
			cText += Chr(xBUFF.GetByte(nInd))
		EndFor
		If Upper(cWinText) $ Upper(OEM(cText))
			Exit
		EndIf
		nThehWND = GetWindow(nThehWND, GW_HWNDNEXT)
	EndDo
RETURN nThehWND
// EOP: FindChildByText
   
/*
    FindVdBWin -- finds a Visual dBASE Window
    Calls FindChildByClass and FindChildByText
    by Jim Sare
    Posted by Romain Strieff in the dBASE2000 newsgroups
*/
FUNCTION FindVdBWin(cWindowCaption)

	LOCAL nThehWND

	nThehWND = FindChildByClass;
   (_app.FrameWin.hWND, "MDICLIENT")
	If nThehWND = 0
		RETURN 0					
	EndIf
RETURN FindChildByText;
   (nThehWND, cWindowCaption)	// # 0 on success, 0 otherwise
// EOP: FindVdBWin


/* ----------------------------------------------------------------
   GetCD() -- returns drive letter for CD drive

    Posted by Romain Strieff in the Borland VdBASE Newsgroups
   ---------------------------------------------------------------- */
FUNCTION GetCD
   if type( "GetDriveType" ) # "FP"
      extern CINT GetDriveType( CSTRING ) kernel32;
             from "GetDriveTypeA"
   endif
   cRet = ""
   for n = 65 to ( 65+25 ) // a to z
       c = chr(n)
       if GetDriveType( c+":\" ) = 5 // CD is drive type 5
          cRet = C
          exit
       endif
   next
RETURN ( cRet )
// eof: GetCD()


/*
   GetImageName
   Gets the name of an image that has been dropped
   into the Image Viewer in Visual dBASE (i.e., 
   someone drags and drops the image into dBASE, this will
   return the "dataSource")

   Posted by Romain Strieff in the dBASE2000 newsgroups,
   April 13, 1999

   This one relies heavily on:
   FindChildByClass
   FindChildByText
   FindVdBWin
*/
Function GetImageName
  if type( "SendMessagePtr" ) # "FP"
     extern cLogical  SendMessagePtr;
     (cHandle, cint, cWord, cString) user32 ;
                      from "SendMessageA"
  endif
  #define WM_GETTEXT htoi("D")
  nH= FindVdBWin("IMAGE VIEWER")
  cBuffer=space(128) + chr(0)

  SendMessagePtr(nH,WM_GETTEXT,128,cBuffer)

return left(cBuffer,at(" - ",cBuffer))
// end of function: GetImageName()


/*
   GetInternetConnection

   Posted by Andrew Shimmin in the dBASE newsgroups,
   January 11, 2010

*/
function getInternetConnection
   //
   // Method to check for an available connection to the internet.
   //
   // Input:
   //
   //    nResultMask =  return result flags mask
   //
   // Output:
   //
   //    true/false  =  available internet connection/NO available 
   //                   internet connection
   //
   // Notes:
   //
   //    nResultMask can have the following values:
   //
   #define INTERNET_CONNECTION_CONFIGURED    0x40
   // Local system has a valid connection to the Internet, 
   // but it might or might not be currently connected.
   #define INTERNET_CONNECTION_LAN           0x02
   // Local system uses a local area network to connect to the Internet.
   #define INTERNET_CONNECTION_MODEM         0x01
   // Local system uses a modem to connect to the Internet.
   #define INTERNET_CONNECTION_MODEM_BUSY    0x08
   // No longer used.
   #define INTERNET_CONNECTION_OFFLINE       0x20
   // Local system is in offline mode.
   #define INTERNET_CONNECTION_PROXY         0x04
   // Local system uses a proxy server to connect to the Internet.

   parameter nResultMask

   if type([InternetGetConnectedState]) # [FP]
      extern clong InternetGetConnectedState(clong,clong) Wininet
   endif

   nResultMask = 0

return (InternetGetConnectedState(nResultMask,0) <> 0)





function GetLabel(cRoot)
/*
   -----------------------------------------------------------------------
   Programmer..: Fred van der Meulen
   Date........: May 1, 2002
   Notes.......: Used to get the label from a disk
   Written for.: dBl
   Rev. History: May 1, 2001 0.1 - Original version 
   Calls.......: None
   Called by...: Any
   Usage.......: GetLabel(<cArg>)
   Example.....: cLabel = GetLabel( "a:\" )
   Returns.....: Disklabel
   Parameters..: cArg = Disk from which the label should be retrieved
   -----------------------------------------------------------------------
*/
   if type( "GetVolumeInformation" ) # "FP"
      extern cLogical GetVolumeInformation;
         (CSTRING, CSTRING, CINT, cptr culong,;
         CINT, CINT, CSTRING, CINT) kernel32;
         from "GetVolumeInformationA"
   endif
   cLabel = space(256)
   nSize = 256
   GetVolumeInformation( cRoot, cLabel, nSize, null, null, null, null, null )
return (cLabel)

function GetPosition
/*
   GetPosition() -- posted by Romain Strieff on the
   Visual dBASE newsgroups, February 28, 1999

   This will return the mouse position anywhere on the
   screen, not just a form or image ...

   Returns character coordinate point values (you
   can simply use the val() function to get the numeric
   values)

   ? GetPosition()

*/
   local pnt,x,y  

   if type("ScreenToClient") # "FP"  
      extern clogical ScreenToClient(chandle, cptr) USER32
   endif

   if type("GetCursorPos") # "FP"
      extern CVOID GetCursorPos ( CPTR )  USER32
   endif 

   // here we check where the mouse cursor is exactly

   pnt = space(8)
   getcursorpos( pnt )   // returns point structure in screencoordinates
     

   x = pnt.GetByte(0) +;
       BitLShift(pnt.GetByte(1), 8) +;
       BitLShift(pnt.GetByte(2), 16) +;
       BitLShift(pnt.GetByte(3), 24)
   y = pnt.GetByte(4) +;
       BitLShift(pnt.GetByte(5), 8) +;
       BitLShift(pnt.GetByte(6), 16) +;
       BitLShift(pnt.GetByte(7), 24)

return return (x+","+y)
// -- eof: GetPosition()

function GetPositionInForm
/*
   GetPositionInWindow() -- posted by Romain Strieff on the
   Visual dBASE newsgroups, February 28, 1999, this is
   a modification posted March 1, 1999 that will return
   the position of the mouse relative to a form/window.

   Returns character coordinate point values (you
   can simply use the val() function to get the numeric
   values)

   Must be called from a form:

   ? GetPositionInForm()

*/
   local pnt,x,y  

   if type("ScreenToClient") # "FP"  
      extern clogical ScreenToClient(chandle, cptr) USER32
   endif

   if type("GetCursorPos") # "FP"
      extern CVOID GetCursorPos ( CPTR )  USER32
   endif 

   // here we check where the mouse cursor is exactly

   pnt = space(8)
   getcursorpos( pnt )   // returns point structure in screencoordinates

   // new code:
//   if type( "ScreenToClient" ) # "FP"
//      extern cLogical ScreenToClient( cHandle, cPtr ) USERDLL32
//   endif
   screenToClient( form.hwnd, pnt ) // relative to current form     

   x = pnt.GetByte(0) +;
       BitLShift(pnt.GetByte(1), 8) +;
       BitLShift(pnt.GetByte(2), 16) +;
       BitLShift(pnt.GetByte(3), 24)
   y = pnt.GetByte(4) +;
       BitLShift(pnt.GetByte(5), 8) +;
       BitLShift(pnt.GetByte(6), 16) +;
       BitLShift(pnt.GetByte(7), 24)

return (x+","+y)
// -- eof: GetPositionInForm()

function GetSerialNumber
   /*
      ---------------------------------------------------------------
      Posted by Romain Strieff in the Visual dBASE Newsgroups
      May 29, 1999

      Updated from a posting in the dBASE, Inc. newsgroups,
      bug-reports newsgroup specifically, by Rich (?)
      January 16, 2002 -- fixes problem with values returned
      on NTFS drives.

      This returns the serial number on a disk drive, and can be
      used to add security to an application, or whatever ...

      Note that this assumes the current drive ...

      Usage: GetSerialNumber()
      ---------------------------------------------------------------
   */

   if type( "WINAPI_GetSerialNumberB" ) # "FP"
      extern cLogical WINAPI_GetSerialNumberB;
             (CSTRING, CSTRING, CINT, cptr culong,;
              CINT, CINT, CSTRING, CINT) kernel32;
             from "GetVolumeInformationA"
   endif
   nBuffer = 0
   WINAPI_GetSerialNumberB( null, null, null, nBuffer, null, null, ;
                           null, null )
return stuff(right("00000000" + itoh(nBuffer),8), 5, 0, '-')
// eof: GetSerialNumber()

function GetSystemName
   /*
      ---------------------------------------------------------------
      Posted by Jim Sare in the Visual dBASE Newsgroups
      February 11, 1999

      This uses api calls to obtain the "computer name".
      This was posted in response to a suggestion to obtain
      this information from the Registry, and Jim pointed
      out that the next release of the OS (Win 2000, or 
      whatever) could change the registry key ...

      Usage:
         ? GetSystemName()
      ---------------------------------------------------------------
   */
   local xBuff, nLen

   if type( "GetComputerName" ) # "FP"
      extern CLOGICAL GetComputerName( CSTRING, CPTR CULONG ) kernel32;
             from "GetComputerNameA"
   endif

   #define MAX_COMPUTERNAME_LENGTH 15

   nLen = MAX_COMPUTERNAME_LENGTH + 1

   xBuff = replicate( chr(0), nLen )
   if GetComputerName( xBuff, nLen )
      RETURN left( xBuff.string, nLen )
   endif

RETURN ""
// eof: GetSystemName()

FUNCTION HiddenDOS(cPIFName)
/* ---------------------------------------------------------------------
   Programmer..: Jim Sare (jimsare@ameritech.net)
   Date........: 04/11/1998
   Notes.......: Use to launch hidden DOS applications including batch
               : (.BAT) files.
   Written for.: dBASE for Windows/Visual dBASE (all versions)
   Rev. History: 04/14/1996 -- Original
               : 04/11/1998 -- Initial adaptation for 32-bit.
   Calls.......: None
               : WINAPI: WinExec
   Called by...: Any
   Usage.......: HiddenDOS(cPIFName)  (Also see notes below)
   Example.....: HiddenDOS("DOSApp.PIF")
   Returns.....: Numeric < 32 if error, > 31 if successful.
   Parameters..: cPIFName = The name of the PIF file used to launch
                 the DOS application
  ---------------------------------------------------------------------
  
   Extended Usage Notes:
  
   Use the Windows PIF editor to create a PIF (Program Inf. File) to
   define the DOS application runtime parameters. In the PIF file, the
   following should be set:
  
   Program Filename:  Name of the DOS application/batch file to be run.
   Window Title:      Text to appear in the DOS window titlebar.
   Optional Parameters:Any parameters that must be passed to the DOS app.
   Start-up Directory:  The location of the DOS app.
   Display Usage:    Must be set to 'Windowed'. Full-screen DOS sessions
                         cannot be hidden.
   Execution:            Background.
   Close Window On Exit: Checked.
  
   See the PIF Editor's online help for other PIF field settings and
   for further information.
  
   Usage: Assume that the Window Title in the PIF file is 'The DOS App':
  
               nRet = HiddenDOS("DOSApp.PIF")
               If nRet > 31  && DOS application was successfully started
                  Do While FindApp("The DOS App") # 0 && DOS app running
                     Sleep 5
                  EndDo
               Else
                  // DOS application launch failed
               EndIf
   
   Return values as defined by Windows API:
   0   = Out of memory or resources.
   2   = File not found.
   3   = Path not found.
   11  = Invalid .EXE file (non-Windows .EXE or error in .EXE image).
   >31 = Application launch successful. Does not necessarily indicate
         that the application successfully completed its task. This only
         indicates that Windows was able to sucessfully start it.
*/
	If Type("WinExec") # "FP"
		extern CWORD WinExec(CSTRING, CUINT) Kernel32
	EndIf
		#define SW_HIDE		0
	RETURN WinExec(cPIFName + Chr(0), SW_HIDE)
// EOP: HiddenDOS


/*
   ---------------------------------------------------------------- 
   IsCBDropped()
      This should be attached to a combobox as a method
      The purpose is to return a true/false value if
      a combobox has "dropped" from the entryfield.

      Code posted by Romain Strieff in Visual dBASE newsgroups,
      January 12, 1999

   ALSO see custom Combobox (kmCustCombobox) in FORMCNTL.CC
   ---------------------------------------------------------------- 
*/
function isCBDropped
   #define WM_USER            htoi("400")
   #define CB_GETDROPPEDSTATE (WM_USER+23)
   if type( "SendMessage" ) # "FP"
      extern cLogical SendMessage( cHandle, cInt, cWord, cLong );
             user32 from "SendMessageA"
   endif

return SendMessage( this.hwnd, CB_GETDROPPEDSTATE, 0, 0 )
// eof: isCBDropped()

/*
    IsExeOpen()
      Posted by Romain some time ago in the dBASE newsgroups,
      this may work in places where isRunning does not.
*/
Function IsExeOpen(cfilename)
   local lRet
   lRet = false // small add
   try
      fClose( fOpen( cFilename, "RW" ) )
   catch ( Exception e ) 
      if e.code == 108 //File in use by another
         lRet := true
      else //file or directory does not exist etc
         msgbox( e.message, "Error" )
         lRet := false
      endif
   endtry
return lret
// eof: IsExeOpen()

/*
    IsReadOnly()
      used to determine if a file's read-only attribute 
      has been set.

      Parameter is the filename:
         ? IsReadOnly( "MyFile.txt" )

      Code posted by Gary White in dB2K programming newsgroup
      May, 2001
*/
function IsReadOnly(cFile)
   #define FILE_ATTRIBUTE_READONLY         0x00000001
	if type('GetFileAttributes') # 'FP'
   	extern culong GetFileAttributes(cstring) kernel32 ;
      	from 'GetFileAttributesA'
   endif
return bitand(GetFileAttributes(cFile), FILE_ATTRIBUTE_READONLY) > 0
// eof: IsReadOnly()

/*
   ----------------------------------------------------------------
   A quick and dirty check to see if an external
   Windows program is executing.
   Returns a logical true/false.

   Based on a posting on the Inprise Newsgroups
   by Angelo Bocchini, January 24, 1999.

   Usage:
      do while isRunning( "wordpad.exe" )
         sleep 2
      enddo
   ----------------------------------------------------------------
*/

function isRunning( cProg )

   if type( "GetModuleHandle" ) # "FP"
      extern CHANDLE GetModuleHandle( CSTRING ) kernel32 ;
             from "GetModuleHandleA"
   endif

return ( GetModuleHandle( cProg ) > 0 ) 
// eof: isRunning()

/*
    isURLGood()
    Small routine that checks to see if a URL is
    valid, before sending a user to it in a browser,
    for example. 

    Example:
    ? isURLGood( "http://www.somedomain.com/index.htm" )

    Posted by Rich Muller in the dBASE Newsgroups,
    August, 2006
*/
function isURLGood(URL)
   local cFile, hResult, lResult
   if type('URLDownloadToFile') # "FP"
      extern culong URLDownloadToFile(cptr,cstring,cstring,culong,;
         culong) URLMON.DLL  from "URLDownloadToFileA"
   endif
   cFile = fUnique()
   // this is the trick: download the URL to a temporary
   // file. 
   hResult = itoh(URLDownloadToFile(null,URL,"&cFile.",null,null))
   // If the temp file exists, we have a good URL
   lResult= iif(new file().exists("&cFile.") and hResult=0,true,false)
   // delete it ...
   new file().delete("&cFile.")
return lResult
// eof: isURLGood()

function makeDirTree
   parameters cDirTree
   // Rich www.autotraker.com
   // This function will build an entire directory tree
   // just pass the entire tree and it will build it.
   // returns true if the directory exists if it has build it.
   // cDirTree = "c:\temp2\temp2\temp3\temp4\temp5\temp6"
   // makeDirTree(cDirTree)
   // can use either imageHlp.dll or dbgHelp.dll
   // (in the system32 directory). Since some versions of
   // imageHlp.dll may not support this function, we are
   // putting it in a try, catch
   local cDir
   cDir = iif(right(cDirTree,1)=="\",cDirTree,cDirTree+"\")
   if type("MakeSureDirectoryPathExists")<>"FP"
      try
         extern cLogical MakeSureDirectoryPathExists(cString);
                "IMAGEHLP.DLL"
      catch(exception e)
      endtry
   endif
   if type("MakeSureDirectoryPathExists")<>"FP"
      extern cLogical MakeSureDirectoryPathExists(cString);
             "dbgHelp.dll"
   endif
return MakeSureDirectoryPathExists(cDir)
// eof: MakeDirTree()

/*
   ------------------------------------------------------------------
   Turn off System Close menu on a form (or the application):

   To use, try:
      NoSysClose( oForm )  (oForm = form object reference)   or
      NoSysClose( _app.FrameWin )


    Posted by Romain Strieff in the Borland VdBASE Newsgroups
   ------------------------------------------------------------------
*/
PROCEDURE NoSysClose
  // greys out and disables the close menuitem
  parameters oForm

  if type( "GetSystemMenu" ) # "FP"
     extern cint GetSystemMenu( cHandle, cInt ) USER32
  endif
  if type( "ModifyMenu" ) # "FP"
     extern cint ModifyMenu( cHandle, cInt, cInt, cInt, cString ) ;
                 USER32;
                 from "ModifyMenuA"
  endif

  hMenu = GetSystemMenu( oForm.hWnd, 0 )
  lSuccess = ModifyMenu( hMenu, 61536, 1, -10, "Close" )
// eof: NoSysClose()

/*
   ------------------------------------------------------------------
   Turn off System Close button on a form (or the application):

   To use, try:
      NoSysCloseBtn( oForm )  (oForm = form object reference)   or
      NoSysCloseBtn( _app.FrameWin )


    Posted by Neil Steven Garber in the Borland VdBASE Newsgroups

    NOTE: This and the above (NoSysClose) appear to be doing
    the same thing in slightly different ways -- also notice the
    different definitions of GetSystemMenu -- these may be 
    incompatible!
   ------------------------------------------------------------------
*/
PROCEDURE NoSysCloseBtn
   parameters oForm

   if type( "GetSystemMenu") # "FP"
      extern CHANDLE GetSystemMenu( cHandle, cLogical ) USER32
   endif

   if type( "DestroyMenu") # "FP"
      extern CINT DestroyMenu( cHandle ) USER32
   endif

   hMenu = GetSystemMenu( oForm.hWnd, false )
   if hMenu > 0
      DestroyMenu( hMenu )
   endif
// eof: NoSysCloseBtn()

* ---------------------------------------------------------------- 
*      OpenURL -- opens a browser with the URL you pass to it.
*      Posted by Romain Strieff on the VdBASE Newsgroups
*      Updated January 12, 1999
*      This version should work for both 5.x and 7.x
*          (which is why the old-style comments)
*      Usage:
*        OpenURL( "someurl" )
*      Example:
*        OpenURL( "http://www.dbase.com" )
*           or
*        OpenURL( "myhtml.htm" )
*  ---------------------------------------------------------------- 
function OpenURL( cURL )
   #define SW_SHOWNORMAL     1
   #define SW_SHOWMINIZED    2
   #define SW_SHOWMAXIMIZED  3
   #define SW_SHOWNOACTIVATE 4

   #define WINAPI_A "A"
   #define WINAPI_SHELL "SHELL32"

   if type( "ShellExecute" ) # "FP"
      extern cHandle ShellExecute( cHandle, cString, cString, ;
             cString, cString, cINT ) WINAPI_SHELL ;
             from "ShellExecute"+WINAPI_A
   endif

   local nHandle
   private sd         && macro needs private not local
   sd = set( "dire" ) && sometimes it *could* change dirs, so 
                      && we are preventing it ...
   nHandle = ShellExecute( _app.framewin.hwnd, "open", cURL, ;
                          null, null, SW_SHOWNORMAL )
   * if the directory changed:
   if sd # set("dire")
      set dire to "&sd."
   endif
return nHandle
*-- EoF: openURL()

Function Play_midi(cMIDIfile)
/*
    Play_midi -- created by Romain Strieff, posted in the
    VdB Newsgroups July 26, 1999

    Plays a MIDI file ...

    Usage:
      Play_MIDI( "filename.mdi" )
*/
   #define SW_SHOWNORMAL 1
   #define SW_SHOWMINIMIZED 2
   #define SW_SHOWMAXIMIZED 3
   #define SW_SHOWNOACTIVATE 4


   #if __vdb__ >= 7
     #define WINAPI_A "A"
     #define WINAPI_SHELL "SHELL32"
   #else
     #define WINAPI_A ""
     #define WINAPI_SHELL "SHELL" 
   #endif

   if type("ShellExecute") #"FP"
      extern cHandle   ShellExecute(cHandle, cstring, cstring, cstring, ;
                         cstring, CINT) WINAPI_SHELL ;
                          from "ShellExecute"+WINAPI_A
   endif
   local nHandle
   private sd && macro needs private
   sd=set("dire") &&sometimes it _could_ change dirs, so prevent it
   nHandle=ShellExecute(_app.framewin.hwnd,"open",cMIDIfile,null,null,SW_SHOWNORMAL)
   if sd#set("dire")
     set dire to "&sd."
   endif
return nHandle 

/*
   ------------------------------------------------------------------
    PrintHTM( cURL )
    A quick routine to print the contents of an HTML file
    Posted by Romain Strieff in the dBASE, Inc. newsgroups
    March 29. 1999

    Usage:
       Create an HTML document somehow
         A file object, for example:
            fHTM = new File()
            fHTM.create( "output.htm" )
            fHTM.puts( "Company Name".bold().underline() )
            fHTM.close()  
         or you could use HTMLCLAS.CC in this library
       // print it:
       PrintHTM( "output.htm" )
       erase ( output.htm )
   ------------------------------------------------------------------
*/
function PrintHTM( cURL )
   #define SW_SHOWNORMAL 1
   if type( "ShellExecute" ) # "FP"
      extern cHandle ShellExecute( cHandle, cString, cString,;
                                   cString, cString, cInt ) ;
                                   WINAPI_SHELL ;
                                   from "ShellExecuteA"
   endif

   local nHandle
   private sd
   sd = set("dire") // in case directory changes

   // this will print it:
   nHandle = ShellExecute( _app.framewin.hwnd, "print",;
                           cURL, null, null, SW_SHOWNORMAL )

   // check directory and return if necessary:
   if sd # set( "directory" )
      set dire to "&sd."
   endif

   // done
return nHandle
// End of Function: PrintHTM()

/*
   ------------------------------------------------------------------
   PrntScrn( <nType> )
   PrntScrn is a quick API call that can be used to send
   the Alt+PrintScrn keystroke to the system. There are
   two different options for this, one for fullscreen (0),
   and the other is for the active window ...

   Borrowed from a posting by Romain Strieff in the Visual
   dBASE newsgroups Sept. 14, 1998

   Usage:
      PrntScrn( 0 ) // full screen
        or
      PrntScrn( 1 ) // active window
   ------------------------------------------------------------------
*/
function PrntScrn( nType )
   // fixed based on note by Arthur Sanfins in the newsgroups,
   // April 29, 2002, missing parens on pCount.
   if pCount() = 0
      nType = 1 // full screen for default
   endif
   if nType < 0 or nType > 1
      msgbox( "Must be either a zero (full screen), or a 1 (active window)",;
              "Bad parm for PrintScrn call!" )
      return
   endif

   if type( "keybd_event" ) # "FP"
      extern cVoid keybd_event( cInt, cInt, cWord, cWord) user32
   endif
   #define VK_SNAPSHOT htoi("2C")
   keybd_event( VK_SNAPSHOT,0,0,nType)
return
// -- eof: PrntScrn()

/*
    Reboot()
    Sept. 16, 1999
    Gary White

    Forces a reboot of a computer ...
*/
function reboot
   #define EWX_REBOOT  2
   #define EWX_FORCE   4
   if type( "ExitWindowsEx" ) # "FP"
      extern clogical ExitWindowsEx( CUINT, CULONG ) USER32
   endif
   ExitWindowsEx( bitor( EWX_REBOOT, EWX_FORCE ), 0 )

// end of function: reboot()

/*
   ------------------------------------------------------------------
   RD()
   01/02/1998
   Romain Strieff

   Visual dBASE does not include an RD command/function,
   so Romain got into the API and discovered how to add
   that functionality.

   Just like the DOS RD command, the directory must be
   empty, and this function will return an error if
   it cannot delete the directory (it doesn't tell you
   if the directory doesn't exist or if the directory
   is empty ...).

   USAGE:
      rd( "somedir" )
         or
      if not rd( "somedir" )
         msgbox ...)
      endif

   ------------------------------------------------------------------
*/
function RD( cDirectory )

   if type( "RmDir" ) # "FP"
      extern cLogical RmDir( cString ) kernel32;
             from "RemoveDirectoryA"
   endif

RETURN RmDir( cDirectory )
// eof: RD()

FUNCTION RenameDirectory(cFrom, cTo)
   * RenameDirectory('c:\the old path', 'c:\the new path')
   * Gary White
   * Give the full paths.
   * Changes intermediate paths, and multi-stage changes,
   *  but cannot change drive.
   * Not reliable with multiple moves.

   if type('MoveFile') # 'FP'
      extern clogical MoveFile(cstring, cstring) kernel32 ;
         from 'MoveFileA'
   endif

return MoveFile(cFrom, cTo)
// EoF: RenameDirectory()

function RunHidden(cDOScommands,cBatchfile)
/*
// By Peter Rorlick
//
// Modified by Chris Zoumas for the array support (cDOScommands must be an one
// dimensional array or else the previous version will execute)
//
// Technique suggested by Romain Strieff
// Example: RunHidden("copy test1.txt LPT1"+chr(13)+chr(10)+"copy test2.txt LPT1")
*/
   #define SW_HIDE         0
   local nCommands, cType, nMessage
   if((argCount()<2).or.(empty(cBatchfile)))
     cBatchfile = "Junk.bat"
   endif
   //
   // Find the number of commands to enter in the batch file ......................
   cType = type("Argvector(1)")
   if(cType = "A")
     nCommands = ALEN(cDOScommands,1)
   elseif(cType = "C")
      if(empty(cDOScommands))
         nMessage=msgbox("Empty Command. Operation Abandoned.","Alert",0+16)
         return
      endif
      nCommands = 1
   else
      nMessage=msgbox("False Command format. Operation Abandoned.","Alert",0+16)
      return
   endif

   // Pass commands in the batch file .............................................
   nH = fcreate(cBatchfile)
   for i=1 TO nCommands
       if (cType="A")
          if (not empty(cDOScommands[i]))
             fputs(nH,cDOScommands[i])
          endif
       elseif(cType="C")
          fputs(nH,cDOScommands)
       endif
   endfor
   fclose(nH)

   if (type("WinExec") # "FP")
      extern CWORD WinExec(CSTRING, CUINT) Kernel32
   endIf

   WinExec(cBatchfile+Chr(0), SW_HIDE)
return
// -- eof: RunHidden()

function RunIt( cProg, nState )
/*
   ------------------------------------------------------------------
   A quick routine posted by Romain Strieff on the
   Visual dBASE newsgroups, January 29, 1999

   This will run an application in whatever state
   you need.

   Usage:
      runIt( "MyProgram", 2 ) // minimized

      Parameters:  cProg = name of program to execute
                   nState = state: 1 = Normal
                                   2 = Minimized
                                   3 = Maximized
                                   4 = ShowNoActivate (??)
   ------------------------------------------------------------------
*/
   if type( "ShellExecute" ) # "FP"
      extern cHandle ShellExecute( cHandle, cString, cString,;
                                   cString, cString, cInt );
                                   shell32 from "ShellExecuteA"
   endif

   local nHandle
   private sd // macro needs private, not local
   sd = set("dire") // running something might change directory
   nHandle = ShellExecute( _app.framewin.hwnd, "open",;
                           cProg, null, null, nState )

   if sd # set("dire" ) // did directory change?
      set dire to "&sd."
   endif

return nHandle
// eof: runIt()

/*
    RUN_AVI()
    Posted by Romain Strieff in the Visual dBASE Newsgroups,
    July 17, 1999
    Example: Run_avi("C:\myavis\theavi.avi")
*/
Function RUN_AVI(cAVI)
   #if __vdb__ >= 7
     #define WINAPI_A "A"
     #define WINAPI_SHELL "SHELL32"
   #else
     #define WINAPI_A ""
     #define WINAPI_SHELL "SHELL" 
   #endif

   if type("ShellExecute") #"FP"
      extern cHandle   ShellExecute(cHandle, cstring, cstring, cstring, ;
                         cstring, CINT) WINAPI_SHELL ;
                          from "ShellExecute"+WINAPI_A
   endif
   local nHandle
   private sd && macro needs private
   sd=set("dire") &&sometimes it _could_ change dirs, so prevent it
   nHandle=ShellExecute(_app.framewin.hwnd,"open",cAVI,null,null,1)
   if sd#set("dire")
     set dire to "&sd."
   endif
return nHandle 
// eof: Run_AVI()

function SetLabel(cRoot,cLabel)
/*
   --------------------------------------------------------------------
   Programmer..: Fred van der Meulen
   Date........: May 1, 2002
   Notes.......: Used to label a disk
   Written for.: dBL
   Rev. History: May 1, 2001 0.1 - Original version 
   Calls.......: None
   Called by...: Any
   Usage.......: SetLabel(<cArg1>,<cArg2>)
   Example.....: SetLabel( "a:\","LabelA" )
   Parameters..: cArg1 = Disk to set a label on
               : cArg2 = Label
   --------------------------------------------------------------------
*/
   if type( "SetVolumeLabel" ) <> "FP"
      extern CLOGICAL SetVolumeLabel( cstring,cstring ) kernel32;
         from "SetVolumeLabelA"
   endif
   SetVolumeLabel( cRoot,cLabel )
return
// eof: SetLabel()

/*
   ------------------------------------------------------------------
   SetReadOnly    
   Set's read-only attribute on or off, depending on
   second parameter, for a specific filename
   Posted by Gary White on the dBASE, Inc. newsgroups

   Example: SetReadOnly( "myFile.txt", 1 ) // ReadOnly
            SetReadOnly( "myFile.txt", 0 ) // Editable
   ------------------------------------------------------------------
*/
function SetReadOnly(cFileName, bReadOnly)
   local bRet
   if type('GetFileAttributes') # 'FP'
      extern culong GetFileAttributes(cstring) kernel32;
      from 'GetFileAttributesA'
   endif
   if type('SetFileAttributes') # 'FP'
      extern clogical SetFileAttributes(cstring, culong) kernel32;
         from 'SetFileAttributesA'
   endif
   nAttr = GetFileAttributes(cFileName)
   if nAttr < 0xffffffff
      if bReadOnly
         nAttr := bitor(nAttr, 1)
      else
         if bitset(nAttr, 0)
            nAttr := bitxor(nAttr, 1)
         endif
      endif
      bRet = SetFileAttributes(cFileName, nAttr)
   else
       bRet = false
   endif
return bRet
// eof: SetReadOnly()

/*
   ------------------------------------------------------------------
   SendMail -- a routine by Jrgen Feder, posted on the
     VdBASE Newsgroups

   You must be connected to the internet at the time this
   call is made ...

   Parameter is the email address
   ------------------------------------------------------------------
*/
function SendMail( cAddress )
   private cAddr
   if type("WinExec") # "FP"
      extern cINT WinExec( CSTRING, CINT) kernel32
   endif
   cAddr = "mailto:"+cAddress
   WinExec( [start "&cAddr"], 0 )
// eof: SendMail()

Function ShowInspector
//----------------------------------------------------
// Posted by Romain Strieff in the Visual dBASE
// newsgroups, July 24, 1999
//
// Adapted from Jim Sare's Findapp
// Moves the Inspector to the upper left corner if it's 
// hidden in some negative location
// Just run it from the command window
// NOTE: This is not necessary for dB2K ... the inspector
// is self-correcting in this aspect (as are the other
// designer palettes).
// ----------------------------------------------------

	LOCAL nThehWND, cBuff, nLen, cText, nInd
	If Type("GetWindow") # "FP"
		extern CHANDLE GetWindow(CHANDLE, CUINT) User32
	EndIf
   If Type("MoveWindow") # "FP"

      extern CLOGICAL MoveWindow;
      (CHANDLE, CINT, CINT, CINT, CINT, CLOGICAL) user32
   endif

	#define GW_HWNDFIRST	0
	#define GW_HWNDNEXT	2

	If Type("GetWindowText") # "FP"
		extern CINT GetWindowText(CHANDLE, CPTR, CINT) User32;
				From "GetWindowTextA"
	EndIf
	nThehWND = GetWindow;
   (_app.FrameWin.hWND, GW_HWNDFIRST)
	cBuff = Space(81)						
	Do While nThehWND # 0
		nLen = GetWindowText;
      (nThehWND, cBuff, 80)	&& Get window caption
		cText = ""
		For nInd = 0 To nLen - 1
			cText += Chr(cBuff.GetByte(nInd))
		EndFor
		If "INSPECTOR" $ Upper(OEM(cText)) And; 
		   nThehWND # _app.FrameWin.hWND		
			Exit							
			
		EndIf
		nThehWND = GetWindow;
      (nThehWND, GW_HWNDNEXT) && Get next child of desktop
	EndDo
   if nThehWND>0
       MoveWindow(nThehWND,0,0,400,400,true)
   endif
RETURN nThehwnd>0

Function Smallfonts
/*
   Programmer   : Romain Strieff [dBVIPS]
   Last Modified: 04/20/1999
   Rev. History.: 04/20/1999 -- Original
   Notes........: Returns True if Smallfonts are used
   .............: in Windows menus
   Written for..: Visual dBASE (32-bit versions)
   Usage .......: ? SmallFonts()
*/
   local dc,result
   extern CINT        GetDeviceCaps(cHandle, CINT) gdi32
   extern CINT        ReleaseDC(cHandle, cHandle) user32
   extern cHandle     GetDC(cHandle) user32
   #define LOGPIXELSX    88
   dc=getdc(0)
   result=GetDeviceCaps(dc,LOGPIXELSX)
   ReleaseDc(0,dc)
return result=96
// eof: SmallFonts()

function ShutDown
/*
    Shutdown -- shuts Windows down. 
    Posted by Rafael Carrazco in the dBASE.codelib newsgroup
    March 5, 2002
*/
   #define EWX_SHUTDOWN  1
   if type( "ExitWindowsEx" ) # "FP"
      extern clogical ExitWindowsEx( CUINT, CULONG ) USER32
   endif
   ExitWindowsEx( EWX_SHUTDOWN, 0 )
return

function LogOff
/*
    Logoff -- As ShutDown, but rather than a complete
              shutdown of Windows, it logs the current
              user's session off the machine.
    Posted by Rafael Carrazco in the dBASE.codelib newsgroup
    March 5, 2002
*/

   #define EWX_POWEROFF  8
   if type( "ExitWindowsEx" ) # "FP"
      extern clogical ExitWindowsEx( CUINT, CULONG ) USER32
   endif
   ExitWindowsEx( EWX_POWEROFF, 0 )
return

/* 
   ------------------------------------------------------------------
   Enables or disables the "close" item on the system menu as well 
   as the Win95 X.

      Syntax:    sysClose( oForm, lEnabled )
      Example:   sysClose( _app.FrameWin, false ) // disable
                 sysClose( _app.FrameWin, true )  // enable

   Posted by Gary White in the VdBASE newsgroups
   7/3/1998

   Returns the previous state of the menu item.

   Modified based on observation by Marc Hamelin in the dBASE.codelib
   newsgroup, Jan. 27, 2003, that the second parameter to 
   GetSystemMenu should be logical ...
   ------------------------------------------------------------------
*/
function sysClose
   parameters oForm, lEnabled
   local hMenu, lRetVal
   #define MF_ENABLED      0x00000000
   #define MF_GRAYED       0x00000001
   #define SC_CLOSE        0xF060

   if argCount() # 2 or ;
         type("oForm.hWnd") # "N" ;
         or type("lEnabled") # "L"
      msgbox( "Correct syntax is:  sysClose( oForm, lEnabled )", ;
         "Error", 16 )
      return
   endif
   if empty( oForm.hWnd )
      msgbox( "Form is not open", "Error", 16 )
      return
   endif

   if type( "GetSystemMenu" ) # "FP"
     extern cint GetSystemMenu( cHandle, cLogical ) USER32
   endif
   if type( "EnableMenuItem" ) # "FP"
      extern CLOGICAL EnableMenuItem( cHandle, cInt, cInt) user32
   endif
	hMenu = GetSystemMenu( oForm.hWnd, false )
   if lEnabled
   	lRetVal = EnableMenuItem( hMenu, SC_CLOSE, MF_ENABLED )
   else
   	lRetVal = EnableMenuItem( hMenu, SC_CLOSE, MF_GRAYED )
	endif
return not lRetVal
// eof: sysClose()

/*
   TableOpenCount() -- a function by Romain Strieff, posted
   on the Visual dBASE newsgroups, Feb. 26, 1999

   According to Romain, the USE command opens a table
   twice (?). This function (which uses the BDE API)
   will return the number of times a table is
   open, which can be used to deactivate queries and so 
   on ...

   Usage: TableOpenCount( tablename, database.handle )
*/
Function TableOpenCount(cTablename,hDatabasehandle)
   if pcount()<2
      hDatabasehandle= _app.databases[1].handle
   endif
   if type("GetOpenCount") # "FP"
      extern cint GetOpenCount(cHandle,cptr,cptr,cstring) idapi32;
             from "DbiGetTableOpenCount"
   endif
   local cTemp, x, c
   cTemp = Replicate(Chr(0), ((Len(cTablename) + 1) / 2) +;
                             ((Len(cTablename) + 1) % 2))
   For x = 1 To Len(cTablename)
       cTemp.SetByte(x - 1, Asc(SubStr(cTablename, x)))
   EndFor
   c=space(1)
   GetOpenCount(hDatabasehandle,cTemp,null, c)
return asc(c)
// -- EoF: TableOpenCount()

/*
    WindowsDirectory()
    Posted by Gary White in the dBASE newsgroups
    October 11, 1999

    Returns path to the Windows directory (where Windows is installed).

    ? WindowsDirectory()
*/
function WindowsDirectory
   local cWinDir, nLen
   if type('GetWindowsDirectory') # 'FP'
      extern cuint GetWindowsDirectory( cstring, cuint ) kernel32 ;
         from "GetWindowsDirectoryA"
   endif
   cWinDir = space(128)
   nLen = 128
   nLen = GetWindowsDirectory( cWinDir, nLen)
   cWinDir = left( cWinDir, nLen )
return cWinDir
// -- EoF: WindowsDirectory()

/*
    GetWindowsFolders()
    Posted by Christopher Neumann in the dBASETalk newsgroups,
    August, 2004

    Updated: Ken Mayer, to return an associative array, which
             can then be queried ...

    Returns an assocarray of paths of special folders used by
    Windows. 

    To view the contents, you could loop through the array,
    or just use the name as shown in the array below. For example,

    aFolders = GetWindowsFolders()
    ? aFolders[ "Desktop Directory" ]

    or to loop through the list:

    aFolders = GetWindowsFolders()
    cKey = aFolders.firstKey
    for i = 1 to aFolders.count()
        ? cKey + " = " + aFolders[ cKey ]
        cKey = aFolders.nextKey( cKey )
    next
*/
function GetWindowsFolders
   private n, sBuffer, nFolder, aWindowFolder, aFolderNames, cKey

   // Create an array of folder names -- you must use these
   // in the exact case shown below, when searching for
   // a folder, or you will not find it ...:
   aFolderNames = new Array()
   aFolderNames.add( "Desktop" )                  //  0
   aFolderNames.add( "Internet" )                 //  1
   aFolderNames.add( "Programs" )                 //  2
   aFolderNames.add( "Controls" )                 //  3
   aFolderNames.add( "Printers" )                 //  4
   aFolderNames.add( "Personal" )                 //  5
   aFolderNames.add( "Favorites" )                //  6
   aFolderNames.add( "Startup" )                  //  7
   aFolderNames.add( "Recent" )                   //  8
   aFolderNames.add( "Send To" )                  //  9
   aFolderNames.add( "Bit Bucket" )               // 10
   aFolderNames.add( "Start Menu" )               // 11
   aFolderNames.add( "My Documents" )             // 12
   aFolderNames.add( "My Music" )                 // 13
   aFolderNames.add( "My Video" )                 // 14
   aFolderNames.add( "Nothing" )                  // 15
   aFolderNames.add( "Desktop Directory" )        // 16
   aFolderNames.add( "Drives" )                   // 17
   aFolderNames.add( "Network" )                  // 18
   aFolderNames.add( "Nethood" )                  // 19
   aFolderNames.add( "Fonts" )                    // 20
   aFolderNames.add( "Templates" )                // 21
   aFolderNames.add( "Common Start Menu" )        // 22
   aFolderNames.add( "Common Programs" )          // 23
   aFolderNames.add( "Common Startup" )           // 24
   aFolderNames.add( "Common Desktop Directory" ) // 25
   aFolderNames.add( "App Data" )                 // 26
   aFolderNames.add( "Printhood" )                // 27
   aFolderNames.add( "Local App Data" )           // 28
   aFolderNames.add( "Alt Startup" )              // 29
   aFolderNames.add( "Common Alt Startup" )       // 30
   aFolderNames.add( "Common Favorites" )         // 31
   aFolderNames.add( "Internet Cache" )           // 32
   aFolderNames.add( "Cookies" )                  // 33
   aFolderNames.add( "History" )                  // 34
   aFolderNames.add( "Common App Data" )          // 35
   aFolderNames.add( "Windows" )                  // 36
   aFolderNames.add( "System" )                   // 37
   aFolderNames.add( "Program Files" )            // 38
   aFolderNames.add( "My Pictures" )              // 39
   aFolderNames.add( "Profile" )                  // 40
   aFolderNames.add( "SystemX86" )                // 41
   aFolderNames.add( "Program FilesX86" )         // 42
   aFolderNames.add( "Program Files Common" )     // 43
   aFolderNames.add( "Program Files CommonX86" )  // 44
   aFolderNames.add( "Common Templates" )         // 45
   aFolderNames.add( "Common Documents" )         // 46
   aFolderNames.add( "Common Admin Tools" )       // 47
   aFolderNames.add( "Admin Tools" )              // 48
   aFolderNames.add( "Connections" )              // 49
   aFolderNames.add( "Nothing 2" )                // 50
   aFolderNames.add( "Nothing 3" )                // 51
   aFolderNames.add( "Nothing 4" )                // 52
   aFolderNames.add( "Common Music" )             // 53
   aFolderNames.add( "Common Pictures" )          // 54
   aFolderNames.add( "Common Video" )             // 55

   // Prototype the API function:
   if type('SHGetFolderPath') # 'FP'
      extern clong SHGetFolderPath(clong,clong,clong,clong,cstring) shfolder.dll from "SHGetFolderPathA"
   endif

   // this is the assoc array to be returned:
   aWindowFolder = new AssocArray()
   // loop through and get the values needed
   for nFolder = 0 to 55
       sBuffer = space(255)
       n = SHGetFolderPath(null, nFolder, 0, null, sBuffer)
       // get the folder name, so we can use that as the
       // key for the associative array:
       cKey = aFolderNames[ nFolder+1 ]
       // store the current path to a key in the array
       // using the folder name from the list above:
       aWindowFolder[ cKey ] = sBuffer.rightTrim()
   next   

return aWindowFolder
// EoF: GetWindowsFolders()

function getWindowsCSIDLPath(CSIDL,cPath,cDisplayMessages,cCreateFolder)
//
// Method to return the path for a windows folder CSIDL.
//
// This function was written to more fully support Vista than the current
// function GetWindowsFolders() in the dUFLP file miscapi.prg.
//
// Vista replaced the windows CSIDL values with KNOWNFOLDERID values and
// replaced the API SHGetFolderPath() with the API SHGetKnownFolderPath().
// The windows special folders and their meaning were also modified and
// SHGetFolderPath() is now a wrapper for SHGetKnownFolderPath() and CSIDL
// values are mapped to KNOWNFOLDERID values.
//
// Vista also allows users or administrators to redirect a known folder to
// a location that suits their needs.
//
// This is achieved by calling IKnownFolderManager::Redirect, which sets the
// "current" value of the folder associated with the SHGFP_TYPE_CURRENT flag.
// The default value of the folder, which is the location of the folder if a
// user or administrator had not redirected it elsewhere, is retrieved by
// specifying the SHGFP_TYPE_DEFAULT flag.
//
// For more detailed explanation see the following URL:
//
// http://msdn2.microsoft.com/en-us/library/bb762181(VS.85).aspx
//
// Author:  Andrew Shimmin
// Date:    4-Jan-2008
// Version: 0.0.0.1
//
// Usage:
//
//    Inputs:
//
//       CSIDL             =  CSIDL value or string of windows folder   [required]     [default - none]
//       cPath             =  "CurrentPath" or "DefaultPath"            [optional]     [default - "CurrentPath"]
//       cDisplayMessages  =  "DisplayMessages" to display errors       [optional]     [default - no messages displayed]
//       cCreateFolder     =  "CreateFolder" to create the windows
//                            folder if it does not already exist       [optional]     [default - does not create the folder]
//
//       The function's input arguments can be in any order but string arguments MUST following the following rules:
//
//       A CSIDL value string like "CSIDL_COMMON_APPDATA" MUST begin with "CSIDL_".
//       All other string arguments MUST be as defined above.
//
//    Outputs:
//
//       The function will return the CSIDL value folder path or an empty()
//       string on an error.
//
//    Examples:
//
//       cFolderPath = GetWindowsCSIDLPath([CSIDL_COMMON_APPDATA])
//       ?cFolderPath
//
//       cFolderPath = GetWindowsCSIDLPath([CSIDL_COMMON_APPDATA],[DefaultPath],[cDisplayMessages])
//       ?cFolderPath
//
//       CSIDL_COMMON_APPDATA = 35
//       cFolderPath = GetWindowsCSIDLPath([cDisplayMessages],CSIDL_COMMON_APPDATA)
//       ?cFolderPath
//
//    Notes:
//
//       Some CSIDL values return a "received an unexpected return from SHGetFolderPath()" error.
//       I have not been able to fathom why as the SHGetFolderPath() call return overflows dBPlus'
//       numeric data type. Maybe some else knows what is happening. I do not need the folder paths
//       for CSIDL values that return this error.
//

   #define  c2Lines     chr(13)+chr(13)

   local    lError,nArgCount,lCSIDLTypeIsName,lFolderCSIDL,lDisplayMessages,;
            lCreate,lCurrentPath,lDefaultPath,lFolder,cFolderPath

   lError = false                                                          // assume no error
   nArgCount = 0                                                           // assume no arguments
   lCSIDLTypeIsName = true                                                 // assume folder CSIDL name provided

   lFolderCSIDL = false                                                    // assume no CSIDL argument
   lDisplayMessages = false                                                // assume no displayMessage argument
   lCreateFolder = false                                                   // assume no createFolder argument
   lCurrentPath = false                                                    // assume no currentPath argument
   lDefaultPath = false                                                    // assume no defaultPath argument

   lFolder = false                                                         // assume folder CISDL not found

   private  nNameCount,aCSIDLNames,aCSIDLValues,cNamesKey,cValuesKey,Arg,;
            nResult,sBuffer,nFolder,aFolderNames,cKey,cExact,nFolder,cFolder,;
            cCSIDLFolderPath

   nNameCount = 0                                                          // reset folder CSIDL names count
   cNamesKey = []                                                          // reset aCSIDLNames key
   cValuesKey = []                                                         // reset aCSIDLValues key
   nFolder = -1                                                            // reset folder CSIDL number
   cFolder = []                                                            // reset folder CSIDL string
   cCSIDLFolderPath = []                                                   // reset return folder CSIDL path

   // create CSIDL names array

   aCSIDLNames = new AssocArray()
   aCSIDLNames.instanceId = program()

   aCSIDLNames["CSIDL_ADMINTOOLS"] = 48
   aCSIDLNames["CSIDL_ALTSTARTUP"] = 29
   aCSIDLNames["CSIDL_APPDATA"] = 26
   aCSIDLNames["CSIDL_BITBUCKET"] = 10
   aCSIDLNames["CSIDL_CDBURN_AREA"] = 59
   aCSIDLNames["CSIDL_COMMON_ADMINTOOLS"] = 47
   aCSIDLNames["CSIDL_COMMON_ALTSTARTUP"] = 30
   aCSIDLNames["CSIDL_COMMON_APPDATA"] = 35
   aCSIDLNames["CSIDL_COMMON_DESKTOPDIRECTORY"] = 25
   aCSIDLNames["CSIDL_COMMON_DOCUMENTS"] = 46
   aCSIDLNames["CSIDL_COMMON_FAVORITES"] = 31
   aCSIDLNames["CSIDL_COMMON_MUSIC"] = 53
   aCSIDLNames["CSIDL_COMMON_PICTURES"] = 54
   aCSIDLNames["CSIDL_COMMON_PROGRAMS"] = 23
   aCSIDLNames["CSIDL_COMMON_STARTMENU"] = 22
   aCSIDLNames["CSIDL_COMMON_STARTUP"] = 24
   aCSIDLNames["CSIDL_COMMON_TEMPLATES"] = 45
   aCSIDLNames["CSIDL_COMMON_VIDEO"] = 55
   aCSIDLNames["CSIDL_COMPUTERSNEARME"] = 61
   aCSIDLNames["CSIDL_CONNECTIONS"] = 49
   aCSIDLNames["CSIDL_CONTROLS"] = 3
   aCSIDLNames["CSIDL_COOKIES"] = 33
   aCSIDLNames["CSIDL_DESKTOP"] = 0
   aCSIDLNames["CSIDL_DESKTOPDIRECTORY"] = 16
   aCSIDLNames["CSIDL_DRIVES"] = 17
   aCSIDLNames["CSIDL_FAVOURITES"] = 6
   aCSIDLNames["CSIDL_FONTS"] = 20
   aCSIDLNames["CSIDL_HISTORY"] = 34
   aCSIDLNames["CSIDL_INTERNET"] = 1
   aCSIDLNames["CSIDL_INTERNET_CACHE"] = 32
   aCSIDLNames["CSIDL_LOCAL_APPDATA"] = 28
   aCSIDLNames["CSIDL_MYDOCUMENTS"] = 5     // was 12 - replaced with CSIDL_PERSONAL
   aCSIDLNames["CSIDL_MYMUSIC"] = 13
   aCSIDLNames["CSIDL_MYPICTURES"] = 39
   aCSIDLNames["CSIDL_MYVIDEO"] = 14
   aCSIDLNames["CSIDL_NETHOOD"] = 19
   aCSIDLNames["CSIDL_NETWORK"] = 18
   aCSIDLNames["CSIDL_PERSONAL"] = 5
   aCSIDLNames["CSIDL_PRINTERS"] = 4
   aCSIDLNames["CSIDL_PRINTHOOD"] = 27
   aCSIDLNames["CSIDL_PROFILE"] = 40
   aCSIDLNames["CSIDL_PROGRAM_FILES"] = 38
   aCSIDLNames["CSIDL_PROGRAM_FILESX86"] = 42
   aCSIDLNames["CSIDL_PROGRAM_FILES_COMMON"] = 43
   aCSIDLNames["CSIDL_PROGRAM_FILES_COMMONX86"] = 44
   aCSIDLNames["CSIDL_PROGRAMS"] = 2
   aCSIDLNames["CSIDL_RECENT"] = 8
   aCSIDLNames["CSIDL_SENDTO"] = 9
   aCSIDLNames["CSIDL_STARTMENU"] = 11
   aCSIDLNames["CSIDL_STARTUP"] = 7
   aCSIDLNames["CSIDL_SYSTEM"] = 37
   aCSIDLNames["CSIDL_SYSTEMX86"] = 41
   aCSIDLNames["CSIDL_TEMPLATES"] = 21
   aCSIDLNames["CSIDL_WINDOWS"] = 36

   // transpose CSIDL names array to get CSIDL values array

   aCSIDLValues = new AssocArray()
   aCSIDLValues.instanceId = program()

   cNamesKey = aCSIDLNames.firstKey

   for   n = 1 to aCSIDLNames.count()
      cValuesKey = ltrim(str(aCSIDLNames[cNamesKey]))
      aCSIDLValues[cValuesKey] = cNamesKey
      cNamesKey = aCSIDLNames.nextKey(cNamesKey)
   endfor

   // parse arguments
   //
   // The arguments can be in any order in the function call and if they include
   // the cDisplayMessages, cCreateFolder, cCurrentPath or cDefaultPath arguments
   // these MUST be the same character string (case insensitive) as specified
   // in the case Statusments below.
   //
   // The folder CSIDL argument can be either a string or a numeric value. When
   // the folder CSIDL argument is a string value it MUST be preceded by "CSIDL_".

   if argcount() > 0                                                       // any arguments?
      for nArgCount = 1 to argcount()                                      // parse them
         arg = argvector(nArgCount)                                        // public for type()
         do case
            case  type([arg]) == "C"
                  do case
                     case  upper(arg) == [DISPLAYMESSAGES]                 // show messages
                           lDisplayMessages = true
                     case  upper(arg) == [CREATEFOLDER]                    // create folder
                           lCreateFolder = true
                     case  upper(arg) == [CURRENTPATH]                     // current path
                           lCurrentPath = true
                     case  upper(arg) == [DEFAULTPATH]                     // default path
                           lDefaultPath = true
                     otherwise                                             // folder CSIDL string
                           if upper(left(arg,6)) == [CSIDL_]                        // folder CSIDL string?
                              lFolderCSIDL = true                                   // set folder CSIDL provided
                              lCSIDLTypeIsName = true                               // set folder CSIDL type
                              lFolder = aCSIDLNames.isKey(upper(trim(arg)))         // set folder CSIDL found
                              cFolder = trim(arg)                                   // set folder CSIDL string
                              nFolder = iif(lFolder,aCSIDLNames[upper(cFolder)],-1) // set folder CSIDL value
                           endif
                  endcase
            otherwise                                                      // folder CSIDL number
                  if type([arg]) == [N]                                             // folder CSIDL value?
                     lFolderCSIDL = true                                            // set folder CSIDL provided
                     lCSIDLTypeIsName = false                                       // set folder CSIDL type
                     lFolder = aCSIDLValues.isKey(ltrim(str(arg)))                  // set folder CSIDL found
                     nFolder = arg                                                  // set folder CSIDL value
                     cFolder = iif(lFolder,aCSIDLValues[ltrim(str(nFolder))],[])    // set folder CSIDL string
                  endif
         endcase
      next
   endif

   // process CSIDL value

   if lFolder                                                              // valid CSIDL?

      // initialise the API

      if type([SHGetFolderPath]) # [FP]
         extern clong SHGetFolderPath(clong,clong,clong,clong,cstring) shfolder.dll from [SHGetFolderPathA]
      endif

      #define  CSIDL_FLAG_CREATE       (0x8000)
      #define  SHGFP_TYPE_CURRENT      (0x0)
      #define  SHGFP_TYPE_DEFAULT      (0x1)
      #define  S_OK                    (0x0)
      #define  S_FALSE                 (0x1)
      #define  E_INVALIDARG            (0x80070057)

      do case
         case  lCurrentPath
               dwFlags = SHGFP_TYPE_CURRENT                                // set for current path
         case  lDefaultPath
               dwFlags = SHGFP_TYPE_DEFAULT                                // set for default path
         otherwise
               dwFlags = SHGFP_TYPE_CURRENT                                // default current path
      endcase

      sBuffer = space(260)                                                 // 260 (not 255) is the Microsoft max path size

      // get CSIDL value path

      nResult = iif(lCreateFolder,SHGetFolderPath(null,nFolder+CSIDL_FLAG_CREATE,0,dwFlags,sBuffer),;
                                  SHGetFolderPath(null,nFolder,0,dwFlags,sBuffer))  // get folder CSIDL path

      do case                                                              // check return
         case  nResult = S_OK                                              // the folder CSIDL is valid and the folder does exist
         case  nResult = S_FALSE                                           // the folder CSIDL is valid but the folder does not exist
               if lDisplayMessages
                  msgbox([The folder CSIDL]+c2Lines+;
                         "[ "+iif(lCSIDLTypeIsName,cFolder,ltrim(str(nFolder)))+" ]"+c2Lines+;
                         [is valid BUT the folder does NOT exist.],program(),16)
                  lError = true
               endif
         case  nResult = E_INVALIDARG                                      // the folder CSIDL is invalid
               if lDisplayMessages
                  msgbox([The folder CSIDL]+c2Lines+;
                         "[ "+iif(lCSIDLTypeIsName,cFolder,ltrim(str(nFolder)))+" ]"+c2Lines+;
                         [is invalid.],program(),16)
                  lError = true
               endif
         otherwise                                                         // unexpected return
               if lDisplayMessages
                  msgbox([The folder CSIDL]+c2Lines+;
                         "[ "+iif(lCSIDLTypeIsName,cFolder,ltrim(str(nFolder)))+" ]"+c2Lines+;
                         [received an unexpected return from SHGetFolderPath().],program(),16)
                  lError = true
               endif
      endcase

   else

      if lFolderCSIDL                                                      // folder CSIDL provided?
         msgbox([The folder CSIDL]+c2Lines+;
                "[ "+iif(lCSIDLTypeIsName,cFolder,ltrim(str(nFolder)))+" ]"+c2Lines+;
                [is invalid.],program(),16)
      else
         msgbox([Folder CSIDL value or string argument missing.],program(),16)
      endif

      lError = true

   endif

   // finish

   if lError
      if .not.class::IsDbaseRuntime()                                      // ide active?
         nReply = msgbox([An eror has occurred. Do you wish to debug it?],program(),32+4)
         if nReply = 6
            set step on
         endif
      endif
   endif

   release object aCSIDLNames
   aCISDLnames = []
   release object aCSIDLValues
   aCSIDLValues = []

   if lFolder                                                              // folder CSIDL provided?
      cCSIDLFolderPath = iif(lError,[],trim(sBuffer))                      // return folder CSIDL path
   endif

   cFolderPath = cCSIDLFolderPath

   release  nNameCount,aCSIDLNames,aCSIDLValues,cNamesKey,cValuesKey,Arg,;
            nResult,sBuffer,nFolder,aFolderNames,cKey,cExact,nFolder,cFolder,;
            cCSIDLFolderPath

return (cFolderPath)
// EoF: GetWindowsCSIDLPath()

function SizeEx(hfilehandle)
/*
   -------------------------------------------------------------
   Programmer..: Marc Van den Berghen
   Date........: 09-01-05
   Notes.......: Get the correct filesize for very large files.
   Written for.: dBASE Plus
   Example.....: oFile=new file()
				     ofile.open("largefile.txt")	
			        ? SizeEx(ofile.handle)
   Returns.....: numeric
   Parameters..: hfilehandle  = handle to an open file
   -------------------------------------------------------------
*/

	local resultl,resulth
	resulth=0xFFFFFFFF
	extern cuint GetFileSize(chandle,cptr cuint) kernel32
	resultl=GetFilesize(hfilehandle,resulth)

return resulth*0x100000000+resultl
// End of function: SizeEx()

function SysDir
/*
   SysDir()
   Gary White
   Posted in the dBASE newsgroups
   Returns the path to the Windows\System folder, with no
   trailing backslash. Useful for when you need to do 
   something such as call RegSvr32.exe to register an 
   ActiveX control on a client's machine, as this path
   may not be available to the DOS path, and therefore
   Windows may tell you it doesn't exist.

      run( true, SysDir()+[\regsvr32 /s "SomeActiveX.ocx"] )  
*/
   local cSysDir, nLen, nBuffSize
   if type( "GetSystemDirectory" ) # "FP"
      extern cuint GetSystemDirectory( cString, CUINT ) kernel32;
             from "GetSystemDirectoryA"
   endif
   cSysDir = space( 128 )
   nBuffSize = cSysDir.length
   nLen = GetSystemDirectory( cSysDir, nBuffSize )
   if nLen > nBuffSize
      nBuffSize := nLen
      cSysDir := space( nBuffSize )
      nLen = GetSystemDirectory( cSysDir, nBuffSize )
   endif
   if nLen > 0
      cSysDir := cSysDir.left( nLen )
   endif
return (cSysDir)

function SystemFonts
/*
     SystemFonts()
     Returns a character string describing the system
     font size being used by Windows based on the dPI
     settings in Windows:
        Normal Fonts (96dpi)
        Large Fonts (120 dpi)
        Very Large Fonts
        Other
     Based on a posting by Roland Wingerter of code
     by Marc VdB in the German newsgroups.

     This can be used to help design applications
     where some of your users use 'Normal' fonts,
     and others use 'Large' fonts, etc. (If a user
     is set to 'Large Fonts', sometimes text
     will not fit in a control ...)
*/

   private n, cReturn

   if type( "GetSystemMetrics" ) # "FP"
      extern cint GetSystemMetrics(cint) user32
   endif

   n = GetSystemMetrics(4) // sm_cycaption

   do case
      case n == 26
           cReturn = "Normal Fonts (96 dpi)"
      case n == 30 
           cReturn = "Large Fonts (120 dpi)"
      case n == 36
           cReturn = "Very Large Fonts"
      otherwise
           cReturn = "Non-standard Font Size"
   endcase

return cReturn

function ScrollHome( hWnd )
   /*
        ScrollHome()
        by Roger Sauer, posted on the dBASE newsgroups,
        March 22, 2007

        Moves a window that uses scrollbars (such as
        the reportViewer in dBASE) back to the "home"
        position (top/left).

        Usage: for example, with a reportViewer, 
               in code that navigates from page to 
               page, you might use this before
               calling the reportViewer's ref.render():
                  class::ScrollHome( form.reportViewer1.hwnd )
   */

   #define WM_HSCROLL 0x0114
   #define WM_VSCROLL 0x0115
   #define SB_PAGEUP  2
   #define SB_PAGELEFT 2
   if empty( hWnd )
      return
   endif

   if type( "SendMessage" ) # "FP"
      extern cLong SendMessage( cHandle, cInt, cLong, cLong );
             user32 from "SendMessageA"
   endif

   SendMessage( hWnd, WM_VSCROLL, SB_PAGEUP, 0 )
   SendMessage( hWnd, WM_HSCROLL, SB_PAGELEFT, 0 )

return

function windowExist(cWindowname)
/*
   Can be used to see if a Window exists by text in
   titlebar (this does assume you know what the text is) ...
   Posted by Andrew Shimmin in the dBASE Users' Groups
   February 1, 2008

*/
   local nWinHandle
   if type("FindWindow") # "FP"
      extern CHANDLE FindWindow(CSTRING,CSTRING) user32 from "FindWindowA"
   endif
   nWinHandle = FindWindow(0,cWindowName)
return (nWinHandle > 0) 

/*
   -----------------------------------------------------------------------
   End of File: MISCAPI.PRG
   ----------------------------------------------------------------------- 
*/


